#include "dialer.h"

static const char* ports[]= { "Port 1", "Port 2", "Port 3", 0 };

cDialer::cDialer(QWidget *parent, const char *name, bool modal):
	QDialog( parent, name, modal)
{
	cIsDeviceOpen=false;
	setCaption("Dialer");
	setFixedSize(280,105);
	
	combo_port=new QComboBox(this);
	combo_port->insertStrList(ports);
	combo_port->move(20,20);
	combo_port->setFixedHeight(25);
	combo_port->setStyle(WindowsStyle);
	
	le_id=new QLineEdit(this);
	le_id->move(150,20);
	le_id->setFixedHeight(25);
	le_id->setStyle(WindowsStyle);
	le_id->setFocus();
	
	pb_dial=new QPushButton("Whlen",this);
	pb_dial->move(35,65);
	pb_dial->setStyle(WindowsStyle);
	pb_dial->setDefault(TRUE);
	connect(pb_dial, SIGNAL(clicked()), SLOT(startDial()));
	
	pb_cancel=new QPushButton("Abbrechen",this);
	pb_cancel->move(145,65);
	pb_cancel->setStyle(WindowsStyle);
	connect(pb_cancel, SIGNAL(clicked()), SLOT(reject()));
	
	status=new dialDlg(this, 0, TRUE);
	status->setGeometry(0, 0, 180, 80);
	status->setFixedSize(status->size());
	status->hide();
	connect(status, SIGNAL(cancel()), SLOT(dialCanceled()));
	hide();
}

cDialer::~cDialer(void) {
	if (cIsDeviceOpen)
		SD->vClose();
}

void cDialer::openDevice(const char *device) {
	SD=new ocSpeedDragon("/dev/modem");
	SD->vOpen();
	if (SD->bIsOpen()) {
		SD->vSetTimeout(1);
		connectNotifier();
		cIsDeviceOpen=true;
	}
}

void cDialer::closeDevice(void) {
	if (cIsDeviceOpen) {
		SD->vClose();
		delete SD;
	}
	cIsDeviceOpen=false;
}
void cDialer::setDialID(const char* ID, int port) {
	combo_port->setCurrentItem(port-1);
	le_id->setText(ID);
}

void cDialer::showDialog(void) {
	show();
}

void cDialer::startDial(void) {	
	if (cIsDeviceOpen==false)
		return;
	if (strlen(le_id->text())==0) 
		return;
	
	hideDialog();
	status->move(this->mapToGlobal(QPoint(10, 0)));
	status->show();
	qApp->processEvents();
		
	if (sdEvent->isEnabled()) {
		sdEvent->setEnabled(FALSE);
		#ifdef _DEBUG
		cout << "dialing! notifier disabled..." << endl;
		#endif
	}
	SD->vDial(combo_port->currentItem()+1, le_id->text());
		
	sdEvent->setEnabled(TRUE);
	#ifdef _DEBUG
	cout << "notifier enabled..." << endl;	
	#endif		
}

void cDialer::hideDialog(void) {
	hide();
}

void cDialer::connectNotifier(void) {
	sdEvent=new QSocketNotifier(SD->iGetFileDescriptor(), 
			QSocketNotifier::Read);
	connect(sdEvent, SIGNAL(activated(int)), SLOT(dialEvent()));
	sdEvent->setEnabled(FALSE);
	#ifdef _DEBUG
	cout << "constructor: notifier connected..." << endl;
	cout << "constructor: notifier disabled..." << endl;
	#endif
}

void cDialer::dialEvent(void) {
	sdEvent->setEnabled(FALSE);
	#ifdef _DEBUG
	cout << "read event...";
	#endif
	switch (SD->xWaitForDialResult(combo_port->currentItem()+1)) {
		case PortBusy:
			status->hide();
			emit localBusy();
			QMessageBox::warning(this, "Port belegt",
				"Der angegebene Port ist nicht frei!");
			showDialog();
			#ifdef _DEBUG
			cout << "got 'busy'!" << endl;
			#endif
			return;
		case Timeout:
			status->hide();
			emit timeout();
			QMessageBox::warning(this, "Timeout",
				"Die Zeitbeschrnkung wurde\nberschritten!");
			showDialog();
			#ifdef _DEBUG
			cout << "got 'timeout'!" << endl;
			#endif
			return;
		case Done:
			status->hide();
			emit done();
//			showDialog();
			#ifdef _DEBUG
			cout << "got 'done'!" << endl;
			#endif
			return;
		case NoResult:
			emit noResult();
			if (!(status->isVisible())) {
				status->show();
			}
			break;
		case Waiting:
			emit waiting();
			if (!(status->isVisible())) {
				status->show();
			}
			break;
	}
	#ifdef _DEBUG
	cout << "handled!" << endl;
	#endif
	sdEvent->setEnabled(TRUE);
}

void cDialer::dialCanceled(void) {
	status->hide();
	emit userCanceled();
	sdEvent->setEnabled(FALSE);
	#ifdef _DEBUG
	cout << "canceled! notifier disabled..." << endl;
	#endif
}


