#include "qtdial.h"

qtDial::qtDial(QWidget *parent, const char* name) : QMainWindow(parent, name),
	configFile(".qtdragonrc", getenv("HOME")) 
{	
	menubar=menuBar();
	createMenubar();
	
	QFrame *tmp=new QFrame(this);
	tmp->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
	tmp->resize(350,200);
	setCentralWidget(tmp);
	
	table=new QListView(tmp);
	table->move(2,2);
	table->addColumn("Beschreibung", tmp->width()-154);
	table->addColumn("Rufnummer", 150);
	table->resize(tmp->width()-4, tmp->height());
	table->setAllColumnsShowFocus(TRUE);
	table->setStyle(WindowsStyle);
	connect(table, SIGNAL(doubleClicked(QListViewItem*)),
		SLOT(dialCurrentItem()));
	connect(table, SIGNAL(rightButtonClicked(QListViewItem *, const QPoint &, int)),
		SLOT(itemPopupMenu(QListViewItem *, const QPoint &, int)));
		
	setCaption("QtDial");
	resize(table->width(), table->height());
	readRcFile();
}

qtDial::~qtDial(void) {
	writeRcFile();
	dialDlg->closeDevice();
}

void qtDial::initDialerDevice(void) {
	#ifdef _DEBUG
	cout << "init device..." << endl;
	#endif
	dialDlg=new cDialer(table);
	if (configFile.getKey("device", "devicename")==NULL) {
		dialDlg->openDevice("/dev/modem");
	}
	else {
		dialDlg->openDevice(configFile.getKey("device", "devicename"));
	}
	#ifdef _DEBUG
	cout << "init done" << endl;
	#endif
}

void qtDial::insertItem(void) {
	QListViewItem *tmp;
	cEdit *editDlg=new cEdit(this, 0, TRUE, "Neu");
	editDlg->setDescription("\0");
	editDlg->setID("\0");
	if (editDlg->exec()) {
		if (strlen(editDlg->getID())!=0) {
			if (configFile.getKey("dialIDs", editDlg->getID())!=NULL) {
				QMessageBox::warning(this, "Telefonnummer",
					"Diese Telefonnummer ist bereits vorhanden!");
				return;
			}
			tmp=new QListViewItem(table, editDlg->getDescription(),
				editDlg->getID());
			/* append to configFile */
			configFile.setKey("dialIDs", editDlg->getID(), "1");
			configFile.setKey(editDlg->getID(), "description",
				editDlg->getDescription());
		}
	}
	delete editDlg;
}

void qtDial::dialCurrentItem() {
	
	if (table->currentItem()==NULL)
		return;	
	dialDlg->setDialID(table->currentItem()->text(1));
	connect(dialDlg, SIGNAL(done()), dialDlg, SLOT(hide()));
	
	dialDlg->showDialog();
	dialDlg->hideDialog();
}

void qtDial::deleteCurrentItem(void) {
	
	if (table->currentItem()==NULL)
		return;
				
	switch (QMessageBox::warning(this, "Lschen", 
		"Wollen Sie diesen Eintrag wirklich lschen?", "Ok", 
		"Abbrechen", 0, 1, 1)) {
		
		case 0:
			/* remove from configFile */
			configFile.delKey("dialIDs", table->currentItem()->text(1));
			configFile.delSection(table->currentItem()->text(1), false);
			delete(table->currentItem());	
			break;
		case 1:
		default:
			break;
	}
}

void qtDial::editCurrentItem(void) {
	if (table->currentItem()==NULL)
		return;
	cEdit *editDlg=new cEdit(this, 0, TRUE, "Eigenschaften");
	editDlg->setDescription(table->currentItem()->text(0));
	editDlg->setID(table->currentItem()->text(1));
	if (editDlg->exec()) {
		/* modify configFile */
		if (configFile.getKey("dialIDs", editDlg->getID())!=NULL) {
			QMessageBox::warning(this, "Telefonnummer",
				"Diese Telefonnummer ist bereits vorhanden!");
			return;
		}
		configFile.delKey("dialIDs", table->currentItem()->text(1));
		configFile.setKey("dialIDs", editDlg->getID(), "1");
		configFile.setSection(table->currentItem()->text(1), editDlg->getID());
		configFile.setKey(editDlg->getID(), "description", editDlg->getDescription());
		table->currentItem()->setText(0, editDlg->getDescription());
		table->currentItem()->setText(1, editDlg->getID()); 		
	}
	delete editDlg;
}

void qtDial::itemPopupMenu(QListViewItem *item, const QPoint &point, int col) {
	QPopupMenu *popup=new QPopupMenu();
	table->setSelected(item, TRUE);
	popup->insertItem("Whlen", this, SLOT(dialCurrentItem()));
	popup->insertSeparator();
	popup->insertItem("Lschen", this, SLOT(deleteCurrentItem()));
	popup->insertSeparator();
	popup->insertItem("Eigenschaften", this, SLOT(editCurrentItem()));
	popup->exec(point);
	delete popup;
}

void qtDial::readRcFile(void) {

	char *key;
	QListViewItem *tmp;

	#ifdef _DEBUG
	printf("\nparsing rc-file:\n");
	#endif
	
	configFile.readRcFile();
	if (configFile.moveSection("dialIDs")==TRUE) {
		while(1) {
			if((key=configFile.moveNext())==NULL)
				break;
			else {
				if (configFile.getKey(key ,"description")==NULL) {
					#ifdef _DEBUG
					cout << "missing section for \"" << key << "\"!\n";
					#endif
					continue;
				}
				tmp=new QListViewItem(table, configFile.getKey(key ,
					"description"), key);
				table->show();
			}
		}
	}
	
	/* get window attributes */
	if ((configFile.getKey("qtdial", "win.width")) && 
		(configFile.getKey("qtdial", "win.height"))) {
		this->resize(atoi(configFile.getKey("qtdial", "win.width")),
			atoi(configFile.getKey("qtdial", "win.height")));
	}
	if (configFile.getKey("qtdial", "col0.width")) {
		table->setColumnWidth(0, atoi(configFile.getKey("qtdial", "col0.width")));
	}
	if (configFile.getKey("qtdial", "col1.width")) {
		table->setColumnWidth(1, atoi(configFile.getKey("qtdial", "col1.width")));
	}	
}

void qtDial::writeRcFile(void) {

	char val[80];
	
	/* set windows attributes */
	sprintf(val, "%i", this->height());
	configFile.setKey("qtdial", "win.height", val); 
	sprintf(val, "%i", this->width());
	configFile.setKey("qtdial", "win.width", val);
	sprintf(val, "%i", table->columnWidth(0));
	configFile.setKey("qtdial", "col0.width", val); 
	sprintf(val, "%i", table->columnWidth(1));
	configFile.setKey("qtdial", "col1.width", val);
	
	configFile.writeRcFile();
}

void qtDial::createMenubar(void) {
	menubar->setFrameStyle(QFrame::NoFrame);
	menubar->setMouseTracking(TRUE);
	
	QPopupMenu *file=new QPopupMenu();
	menubar->insertItem("Datei", file);
	file->setMouseTracking(TRUE);
	file->setStyle(WindowsStyle);
	/*file->insertItem("ffnen");
	file->insertItem("Speichern");
	file->insertSeparator();
	file->insertItem("Optionen");
	file->insertSeparator();*/
	file->insertItem("Beenden", qApp, SLOT(quit()));	
	
	edit=new QPopupMenu();
	menubar->insertItem("Bearbeiten", edit);
	edit->setMouseTracking(TRUE);
	edit->setStyle(WindowsStyle);
	edit->insertItem("Whlen");
	edit->insertSeparator();
	edit->insertItem("Neuer Eintrag", this, SLOT(insertItem()));
	edit->insertItem("Lschen");
	edit->insertSeparator();
	edit->insertItem("Eigenschaften");
	connect(edit, SIGNAL(aboutToShow()), SLOT(editMenuSelected()));
}

void qtDial::resizeEvent(QResizeEvent *event) {
	table->resize(this->width()-4, this->height()-30);
}

void qtDial::editMenuSelected(void) {

	/* disconnect - otherwise disabling doesn't work */	
	edit->disconnectItem(0, this, SLOT(dialCurrentItem()));
	edit->disconnectItem(3, this, SLOT(deleteCurrentItem()));
	edit->disconnectItem(5, this, SLOT(editCurrentItem()));
	if (table->currentItem()==NULL) {
		edit->setItemEnabled(0, FALSE);
		edit->setItemEnabled(3, FALSE);
		edit->setItemEnabled(5, FALSE);
	}
	else {
		edit->setItemEnabled(0, TRUE);
		edit->connectItem(0, this, SLOT(dialCurrentItem()));
		edit->setItemEnabled(3, TRUE);
		edit->connectItem(3, this, SLOT(deleteCurrentItem()));
		edit->setItemEnabled(5, TRUE);
		edit->connectItem(5, this, SLOT(editCurrentItem()));
	}
}

int main(int argc, char *argv[]) {
	
	QApplication DIALER(argc, argv);
	qtDial mw;
	
	DIALER.setStyle(WindowsStyle);
	DIALER.setFont(QFont("helvetica", 12), TRUE);
	DIALER.setMainWidget(&mw);
	
	mw.setStyle(WindowsStyle);
	mw.show();
	qApp->processEvents();
	
	mw.initDialerDevice();
	return DIALER.exec();
}
