#include "update.h"
#include "rcfile.h"
#include "../config.h"

/* assign macro to global variable */
static char path_sx[]=PATH_SX; 

cUpdate::cUpdate(QWidget *parent, const char *name):
	QWidget( parent, name, FALSE)
{
	QFrame *tmp;
	char *devicename;
	
	setCaption("QtFwUpdate");
	setFixedSize(400, 230);
	
	/* frame */
	tmp=new QFrame(this);
	tmp->setFrameStyle(QFrame::Panel | QFrame::Sunken);
	tmp->setFixedSize(400, 185);
	
	/* device */
	le_device=new QLineEdit(this);
	QLabel *l_device=new QLabel("Gertename:", this);
	l_device->move(20, 17);
	le_device->move(120, 20);
	le_device->setFixedSize(150, 25);
	le_device->setStyle(WindowsStyle);
	
	/* firmware */
	le_firmware=new QLineEdit(this);
	QLabel *l_firmware=new QLabel("Firmware-Datei:", this);
	l_firmware->move(20, 72);
	le_firmware->move(120, 75);
	le_firmware->setFixedSize(150, 25);
	le_firmware->setStyle(WindowsStyle);
	le_firmware->setEnabled(FALSE);
	pb_search=new QPushButton("Durchsuchen", this);
	pb_search->setFixedSize(95, 25);
	pb_search->move(290, 75);
	pb_search->setStyle(WindowsStyle);
	connect(pb_search, SIGNAL(clicked()), SLOT(getFileName()));
	
	/* progress bar */
	updateProgress=new QProgressBar(this);
	updateProgress->move(120,140);
	updateProgress->setEnabled(FALSE);
	updateProgress->setFixedSize(150, 25);
	QLabel *l_progress=new QLabel("Transfer-Status:", this);
	l_progress->move(20, 137);
	
	/* buttons */
	pb_ok=new QPushButton("Ok", this);
	pb_ok->move(220, 195);
	pb_ok->setFixedSize(80, 25);
	pb_ok->setStyle(WindowsStyle);
	pb_ok->setEnabled(FALSE);
	connect(pb_ok, SIGNAL(clicked()), SLOT(startUpdate()));
	
	pb_cancel=new QPushButton("Abbruch", this);
	pb_cancel->move(310, 195);
	pb_cancel->setFixedSize(80, 25);
	pb_cancel->setStyle(WindowsStyle);	
	connect(pb_cancel, SIGNAL(clicked()), qApp, SLOT(quit()));
	
	/* get devicename */
	rcFile config(".qtdragonrc", getenv("HOME"));
	config.readRcFile();
	devicename=config.getKey("device","devicename");
	if (devicename!=NULL) {
		setDevice(devicename);
	}
	else {
		setDevice("/dev/modem");
	}
}

void cUpdate::setDevice(const char *name) {
	if (name!=NULL) {
		le_device->setText(name);
	}
}

const char* cUpdate::getDevice(void) {
	return(le_device->text());
}

void cUpdate::setFirmware(const char *file) {
	if (file!=NULL) {
		le_firmware->setText(file);
	}
}

const char* cUpdate::getFirmware(void) {
	return(le_firmware->text());
}

void cUpdate::startUpdate(void) {

	char command[256], kbyte[2048];
	int byteRead, fifofd, firmwarefd, fileSize, progress=0;
	
	if ( (strlen(getDevice())==0) | (strlen(getFirmware())==0) )
		return;
	userEventEnabled(FALSE);
	qApp->processEvents();
	
	/* open firmware file */
	firmwarefd=open(getFirmware(), O_RDONLY);
	if (firmwarefd < 0) {
		userEventEnabled(TRUE);
		QMessageBox::warning(this, "Fehler", "Die Firmwaredatei konnte nicht geffnet werden.\
			\nBitte berprfen Sie die Dateirechte!");
		return;
	} 
	
	/* open device and set baudrate */
	SD=new miniCon(getDevice());
	SD->openDevice();
	if (!(SD->isDeviceOpen())) {
		userEventEnabled(TRUE);
		QMessageBox::warning(this, "Fehler", "Das angegebene Gert konnte nicht geffnet werden.\
			\nBitte berprfen Sie Ihre Eingabe und die Dateirechte!");
		return;
	}
	
	/* check if device is a Speed Dragon */
	if ((!(SD->checkDragon())) && (SD->getLastStatus()==sReady)) {
		userEventEnabled(TRUE);
		QMessageBox::warning(this, "Fehler", "Das angegebene Gert ist kein Speed Dragon!");
		return;
	}
	
	/* send update command if at-interpreter is active */
	if (SD->getLastStatus()==sReady) {
		SD->setTimeout(1);
		SD->sendString("ATUPX");
		if (SD->waitString("Proceed")==0) {
			userEventEnabled(TRUE);
			QMessageBox::warning(this, "Fehler", "Vom Speed Dragon wurde eine unerwartete Antwort\
				\nempfangen. Bitte benachrichtigen Sie den Autor!");
			return;
		}
		SD->setTimeout();
		SD->sendString("y");
	}
	
	/* return if baudrate couldn't be detected */	
	if (SD->getLastStatus()==sUnknownError) {
		userEventEnabled(TRUE);
		QMessageBox::warning(this, "Fehler", "Der Speed Dragon antwortet nicht! Bitte berprfen\
			\nSie den Gertenamen und den korrekten Anschlu.");
		return;
	}	
	updateProgress->setEnabled(TRUE);
	
	/* create fifo */
	mkfifo("/tmp/fwupdate.tmp", S_IRUSR | S_IWUSR);
	
	/* start sx -> reading from fifo */
	sprintf(command, "%s -l 32 /tmp/fwupdate.tmp < %s > %s &", path_sx, getDevice(), getDevice());	
	system(command);
	
	/* get size of firmware file */
	QFileInfo *xFileInfo=new QFileInfo(getFirmware());
	fileSize=xFileInfo->size();
	fileSize=fileSize/1024;
	#ifdef _DEBUG
	cout << "firmware size: " << fileSize << " kB" << endl;
	#endif
	delete xFileInfo;
	updateProgress->setTotalSteps(fileSize);
	
	/* open fifo -> writing firmware */
	fifofd=open("/tmp/fwupdate.tmp", O_WRONLY); 	
	unlink("/tmp/fwupdate.tmp"); /* fifo will be removed when closed */
	
	/* copy firmware file to fifo */
	do {
		byteRead=read(firmwarefd, &kbyte, 2048);
		if (byteRead==0) {
			break;
		}
		write(fifofd, &kbyte, byteRead);
		progress=progress+2;
		updateProgress->setProgress(progress);
		qApp->processEvents();
	} while (1);
	
	::close(fifofd);
	::close(firmwarefd);
	
	updateProgress->reset();
	updateProgress->setEnabled(FALSE);
	userEventEnabled(TRUE);
	setFirmware("\0"); /* clear firmware text entry */
	pb_ok->setEnabled(FALSE);
	SD->closeDevice();	
	
	QMessageBox::information(this, "Firmware Update", "Vorgang abgeschlossen!\
				\nberprfen Sie, ob die Firmware fehlerfrei\
				\nbertragen wurde. Andernfalls wiederholen\
				\nSie den Vorgang.");
}

void cUpdate::userEventEnabled(bool val) {
	if (val) {
		qApp->restoreOverrideCursor();
	}
	else {
		qApp->setOverrideCursor(waitCursor, TRUE);
	}
	pb_search->setEnabled(val);
	pb_ok->setEnabled(val);
	pb_cancel->setEnabled(val);
}

void cUpdate::getFileName(void) {
	QString tmp=QFileDialog::getOpenFileName(getenv("HOME"), "*.bin",  this);
	if (tmp!=NULL) {
		pb_ok->setEnabled(TRUE);
		setFirmware(tmp);
	}
}

int main(int argc, char *argv[]) {
	
	QApplication fwUpdate(argc, argv);
	cUpdate mw;
	
	fwUpdate.setStyle(WindowsStyle);
	fwUpdate.setFont(QFont("helvetica", 12), TRUE);
	fwUpdate.setMainWidget(&mw);
	mw.setStyle(WindowsStyle);
	mw.show();
	return fwUpdate.exec();
}
