//
// sd.cpp
//
// (c) Jan Willamowius, 1998
//
// free for private use - commercial use or commercial distribution
// requires permission from the author
//

#include "sd.hpp"
#include <iostream>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>

#ifdef _MSC_VER
#define vsnprintf	_vsnprintf
#endif

 
ocSpeedDragon::ocSpeedDragon(const string & sDevice, int iBps)
				: oConn(sDevice)
{
	xError = NoError;
	bDebug = false;
	oConn.vSetEcho(bDebug);
	iMajorVer = 0;
	iMinorVer = 0;
	sDeviceName = sDevice;
	iConnBps = iBps;
	oConn.vSetTimeout(40);
}

ocSpeedDragon::~ocSpeedDragon()
{
	if (bIsOpen())
		vClose();
}

void ocSpeedDragon::vSetBps(int iNewBps)
{
	iConnBps = iNewBps;
}

int ocSpeedDragon::iGetBps(void) const
{
	return iConnBps;
}
	
void ocSpeedDragon::vChangeBpsPermanently(int iNewBps)
{
	sprintf(cBuf, "AT&HX%d\r", iNewBps);
	oConn.vSendString(cBuf);
	vClose();
	iConnBps = iNewBps;
	vOpen();
}

void ocSpeedDragon::vChangeBpsTemporary(int iNewBps)
{
	sprintf(cBuf, "AT&HB%d\r", iNewBps);
	oConn.vSendString(cBuf);
	vClose();
	iConnBps = iNewBps;
	vOpen();
}

void ocSpeedDragon::vOpen(void)
{
	char * pBuf = NULL;
	int iOldTimeout = iGetTimeout();
	
	if (oConn.bDeviceLocked())
	{
		xError = DeviceLocked;
		return;
	};

	// autodetect baudrate
	if (iConnBps == 0)
	{
		oConn.vAutoSetBps();
		vSetBps(oConn.iGetBps());
	};

	if (iConnBps == 0)
	{
		// AutoBaud failed
		xError = AutobaudError;
		return;
	};
	oConn.vSetBps(iConnBps);

	oConn.vOpen();
	// check for errors
	if (oConn.bIsOpen())
	{
		// get version
		vSetTimeout(3);
		oConn.vSendString("ATI0\r");
		oConn.vWaitString("Hagenuk", "OK");
		vSetTimeout(iOldTimeout);
		if (oConn.iStringFound() != 1)
		{
			oConn.vClose();
			xError = NotHagenuk;
			return;
		};
		oConn.vWaitString("Revision: ");
		oConn.vReceiveString();
		strcpy(cBuf, oConn.sGetLastString().c_str());
		pBuf = cBuf;
		while ( *pBuf == ' ' ) pBuf++;
		iMajorVer = (int)strtol( pBuf, &pBuf, 10 );
		if ( *pBuf == '.' ) pBuf++;
		iMinorVer = (int)strtol( pBuf, &pBuf, 10 );
		vWaitOkError();
	
		if (!bVersionGE(2, 0))
		{
			cerr << "Firmware zu alt (< 2.0)" << endl;
			xError = OldFirmware;
			oConn.vClose();
		};
	}
	else
		xError = UnknownError;
}

void ocSpeedDragon::vClose(void)
{
	oConn.vClose();
}

bool ocSpeedDragon::bIsOpen(void) const
{
	return oConn.bIsOpen();
}

void ocSpeedDragon::vSetDebug(bool bOn)
{
	bDebug = bOn;
	oConn.vSetEcho(bDebug);
}

void ocSpeedDragon::vSetTimeout(int iNewTimeout)	// seconds
{
	oConn.vSetTimeout(iNewTimeout);
}

int ocSpeedDragon::iGetTimeout(void) const			// seconds
{
	return oConn.iGetTimeout();
}

tSDError ocSpeedDragon::xGetLastError(void)
{
	tSDError xResult = xError;
	xError = NoError;
	return xResult;
}

int ocSpeedDragon::iGetFileDescriptor(void)
{
	return oConn.iGetFileDescriptor();
}

string ocSpeedDragon::sGetFirmwareRevision(void) const
{
	char cBuf1[20];

	sprintf(cBuf1, "%d.%d", iMajorVer, iMinorVer);
	return string(cBuf1);
}

void ocSpeedDragon::vClearCallList(void)
{
	if (bVersionGE(2, 5))
	{
		oConn.vSendString("AT&HP\"95\"\r");
		vWaitOkError();
	}
	else
		xError = OldFirmware;
}

void ocSpeedDragon::vSetAmtsBerechtigungPort(tPort xNo, tAmtsBerechtigung xAmt)
{
	vSetAny("9911/%d=%d", xNo, xAmt);
}

tAmtsBerechtigung ocSpeedDragon::xGetAmtsBerechtigungPort(tPort xNo)
{
	return (tAmtsBerechtigung)atoi(sGetAny("9911/%d", xNo).c_str());
}

void ocSpeedDragon::vSetAmtsBerechtigungPBX(tAmtsBerechtigung xAmt)
{
	vSetAny("9912=%d", xAmt-1);
}

tAmtsBerechtigung ocSpeedDragon::xGetAmtsBerechtigungPBX(void)
{
	return (tAmtsBerechtigung)(atoi(sGetAny("9912").c_str()) + 1);
}

void ocSpeedDragon::vSetGebuehrenfaktorPort(tPort xNo, int iFaktor, int iNachkomma)
{
	if ((iNachkomma == 0) || !bVersionGE(3, 10))
		vSetAny("99211/%d=%d", xNo, iFaktor);
	else
		vSetAny("99211/%d=%d*%d#", xNo, iFaktor, iNachkomma);
}

int ocSpeedDragon::iGetGebuehrenfaktorPort(tPort xNo)
{
	return atoi(sGetAny("99211/%d", xNo).c_str());
}

int ocSpeedDragon::iGetGebuehrenNachkommaPort(tPort xNo)
{
	string oFaktor;

	// erst ab Firmware 3.10
	if ( !bVersionGE(3, 10) )
		return 0;

	oFaktor = sGetAny("99211/%d", xNo);
	if(oFaktor.find("*", 0) == string::npos)
		return 0;

	while(oFaktor.find("*", 0) != string::npos)
	{
#if  (_MSC_VER > 1 && _MSC_VER < 1100) || (__GNUC__ == 2 && __GNUC_MINOR__ < 8)
		// Visual C++ 4.2 and gcc <= 2.7.2 call it remove()
		oFaktor.remove(0, 1);
#else
		// gcc >= 2.8 and egcs call it erase()
		oFaktor.erase(0, 1);
#endif
	};
	return atoi(oFaktor.c_str());
}

void ocSpeedDragon::vSetGebuehrenfaktorPBX(int iFaktor, int iNachkomma)
{
	if ((iNachkomma == 0) || !bVersionGE(3, 10))
		vSetAny("99212=%d", iFaktor);
	else
		vSetAny("99212=%d*%d#", iFaktor, iNachkomma);
}

int ocSpeedDragon::iGetGebuehrenfaktorPBX(void)
{
	return atoi(sGetAny("99212").c_str());
}

int ocSpeedDragon::iGetGebuehrenNachkommaPBX(void)
{
	string oFaktor;

	// erst ab Firmware 3.10
	if ( !bVersionGE(3, 10) )
		return 0;

	oFaktor = sGetAny("99212");
	if(oFaktor.find("*", 0) == string::npos)
		return 0;

	while(oFaktor.find("*", 0) != string::npos)
	{
#if (_MSC_VER > 1 && _MSC_VER < 1100) || (__GNUC__ == 2 && __GNUC_MINOR__ < 8)
		// Visual C++ 4.2 and gcc <= 2.7.2 call it remove()
		oFaktor.remove(0, 1);
#else
		// gcc >= 2.8 and egcs call it erase()
		oFaktor.erase(0, 1);
#endif
	};
	return atoi(oFaktor.c_str());
}

int ocSpeedDragon::iGetSummenzaehlerPort(tPort xNo)
{
	return atoi(sGetAny("992211/%d", xNo).c_str());
}

int ocSpeedDragon::iGetSummenzaehlerPBX(void)
{
	return atoi(sGetAny("992212").c_str());
}

void ocSpeedDragon::vClearSummenzaehlerPort(tPort xNo)
{
	vSetAny("992221/%d", xNo);
}

void ocSpeedDragon::vClearSummenzaehlerPBX(void)
{
	vSetAny("992222");
}

void ocSpeedDragon::vRufumleitungOn(const string & sMSN, tRedirectionType xType)
{
	// CFNR und CFB erst ab Firmware 2.18
	if (((xType == CFNR) || (xType == CFB))
           && (!bVersionGE(2, 18)))
	{
		xError = OldFirmware;
		return;
	}

	vSetAny("9950%d1/%s", xType, sMSN.c_str());
}

void ocSpeedDragon::vRufumleitungOff(const string & sMSN, tRedirectionType xType)
{
	// CFNR und CFB erst ab Firmware 2.18
	if (((xType == CFNR) || (xType == CFB))
           && (!bVersionGE(2, 18)))
	{
		xError = OldFirmware;
		return;
	}

	vSetAny("9950%d0/%s", xType, sMSN.c_str());
}

bool ocSpeedDragon::bGetRufumleitungActive(const string & sMSN, tRedirectionType xType)
{
	// CFNR und CFB erst ab Firmware 2.18
	if (((xType == CFNR) || (xType == CFB))
           && (!bVersionGE(2, 18)))
	{
		xError = OldFirmware;
		return false;
	}

	return (sGetAny("9950%d0/%s", xType, sMSN.c_str()) == "1");
}

void ocSpeedDragon::vSetRufumleitung(const string & sMSN, const string & sNumber, tRedirectionType xType)
{
	// CFNR und CFB erst ab Firmware 2.18
	if (((xType == CFNR) || (xType == CFB))
           && (!bVersionGE(2, 18)))
	{
		xError = OldFirmware;
		return;
	}

	vSetAny("9950%d2/%s=%s", xType, sMSN.c_str(), sNumber.c_str());
}

string ocSpeedDragon::sGetRufumleitung(const string & sMSN, tRedirectionType xType)
{
	// CFNR und CFB erst ab Firmware 2.18
	if (((xType == CFNR) || (xType == CFB))
           && (!bVersionGE(2, 18)))
	{
		xError = OldFirmware;
		return "";
	}

	return sGetAny("9950%d2/%s", xType, sMSN.c_str());
}

void ocSpeedDragon::vRufumleitungCFUOn(const string & sMSN)
{
	vRufumleitungOn(sMSN, CFU);
}

void ocSpeedDragon::vRufumleitungCFUOff(const string & sMSN)
{
	vRufumleitungOff(sMSN, CFU);
}

bool ocSpeedDragon::bGetRufumleitungCFUActive(const string & sMSN)
{
	return bGetRufumleitungActive(sMSN, CFU);
}

void ocSpeedDragon::vSetRufumleitungCFU(const string & sMSN, const string & sNumber)
{
	vSetRufumleitung(sMSN, sNumber, CFU);
}

string ocSpeedDragon::sGetRufumleitungCFU(const string & sMSN)
{
	return sGetRufumleitung(sMSN, CFU);
}

void ocSpeedDragon::vRufumleitungCFNROn(const string & sMSN)
{
	vRufumleitungOn(sMSN, CFNR);
}

void ocSpeedDragon::vRufumleitungCFNROff(const string & sMSN)
{
	vRufumleitungOff(sMSN, CFNR);
}

bool ocSpeedDragon::bGetRufumleitungCFNRActive(const string & sMSN)
{
	return bGetRufumleitungActive(sMSN, CFNR);
}

void ocSpeedDragon::vSetRufumleitungCFNR(const string & sMSN, const string & sNumber)
{
	vSetRufumleitung(sMSN, sNumber, CFNR);
}

string ocSpeedDragon::sGetRufumleitungCFNR(const string & sMSN)
{
	return sGetRufumleitung(sMSN, CFNR);
}

void ocSpeedDragon::vRufumleitungCFBOn(const string & sMSN)
{
	vRufumleitungOn(sMSN, CFB);
}

void ocSpeedDragon::vRufumleitungCFBOff(const string & sMSN)
{
	vRufumleitungOff(sMSN, CFB);
}

bool ocSpeedDragon::bGetRufumleitungCFBActive(const string & sMSN)
{
	return bGetRufumleitungActive(sMSN, CFB);
}

void ocSpeedDragon::vSetRufumleitungCFB(const string & sMSN, const string & sNumber)
{
	vSetRufumleitung(sMSN, sNumber, CFB);
}

string ocSpeedDragon::sGetRufumleitungCFB(const string & sMSN)
{
	return sGetRufumleitung(sMSN, CFB);
}

void ocSpeedDragon::vRufumleitungBK2On(const string & sMSN)
{
	vRufumleitungOn(sMSN, BK2);
}

void ocSpeedDragon::vRufumleitungBK2Off(const string & sMSN)
{
	vRufumleitungOff(sMSN, BK2);
}

bool ocSpeedDragon::bGetRufumleitungBK2Active(const string & sMSN)
{
	return bGetRufumleitungActive(sMSN, BK2);
}

void ocSpeedDragon::vSetRufumleitungBK2(const string & sMSN, const string & sNumber)
{
	vSetRufumleitung(sMSN, sNumber, BK2);
}

string ocSpeedDragon::sGetRufumleitungBK2(const string & sMSN)
{
	return sGetRufumleitung(sMSN, BK2);
}

void ocSpeedDragon::vAllRufumleitungenOff(void)
{
	vSetAny("99509");
}

void ocSpeedDragon::vSetCallerIDSuppression(const string & sMSN, bool xOn)
{
	if (xOn)
		vSetAny("99511/%s", sMSN.c_str());
	else
		vSetAny("99510/%s", sMSN.c_str());
}

bool ocSpeedDragon::bGetCallerIDSuppression(const string & sMSN)
{
	return (sGetAny("99510/%s", sMSN.c_str()) == "1");
}

void ocSpeedDragon::vSetSammelrufEintrag(tPort xNo, bool bEintrag)
{
	vSetPortBool("9952", xNo, bEintrag);
}

bool ocSpeedDragon::bGetSammelrufEintrag(tPort xNo)
{
	return bGetPortBool("9952", xNo);
}

void ocSpeedDragon::vSetPseudoDialtone(tPort xNo, bool bOn)
{
	vSetPortBool("9953", xNo, bOn);
}

bool ocSpeedDragon::bGetPseudoDialtone(tPort xNo)
{
	return bGetPortBool("9953", xNo);
}

void ocSpeedDragon::vSetAnklopfen(tPort xNo, bool bOn)
{
	vSetPortBool("9954", xNo, bOn);
}

bool ocSpeedDragon::bGetAnklopfen(tPort xNo)
{
	return bGetPortBool("9954", xNo);
}

void ocSpeedDragon::vSetDisplayPart(tDisplayPart xPart, bool bOn)
{
	// consistency check
	if ((xPart == OnlyRedirectedMSN) && bOn && bGetDisplayPart(AllMSN))
		vSetDisplayPart(AllMSN, false);
	if ((xPart == AllMSN) && bOn && bGetDisplayPart(OnlyRedirectedMSN))
		vSetDisplayPart(OnlyRedirectedMSN, false);
	if ((xPart == BChannels) && bOn && bGetDisplayPart(ExtendedBChannels))
		vSetDisplayPart(ExtendedBChannels, false);
	if ((xPart == ExtendedBChannels) && bOn && bGetDisplayPart(BChannels))
		vSetDisplayPart(BChannels, false);

	if ( (xPart == ExtendedBChannels) && !bVersionGE(2, 5) )
		return;

	// the actual switching
	vSetAny("9955/%d=%d", xPart, bOn?1:0);
}

bool ocSpeedDragon::bGetDisplayPart(tDisplayPart xPart)
{
	return (sGetAny("9955/%d", xPart) == "1");
}

void ocSpeedDragon::vSetLanguage(tLanguage xLanguage)
{
	vSetAny("9972=%d", xLanguage);
}

tLanguage ocSpeedDragon::xGetLanguage(void)
{
	return (tLanguage)atoi(sGetAny("9972").c_str());
}

void ocSpeedDragon::vSetISDNNet(tISDNNet xISDN)
{
	vSetAny("9973=%d", xISDN);
}

tISDNNet ocSpeedDragon::xGetISDNNet(void)
{
	return (tISDNNet)atoi(sGetAny("9973").c_str());
}

void ocSpeedDragon::vClearPortMSN(tPort xNo)
{
	vSetAny("99901/%d", xNo);
}

void ocSpeedDragon::vClearSammelrufMSN(void)
{
	vSetAny("99902");
}

void ocSpeedDragon::vClearFernprogMSN(void)
{
	vSetAny("99903");
}

void ocSpeedDragon::vSetPortMSN(tPort xNo, const string & sMSN)
{
	vSetAny("99911/%d=%s", xNo, sMSN.c_str());
}

string ocSpeedDragon::sGetPortMSN(tPort xNo)
{
	return sGetAny("99911/%d", xNo);
}

void ocSpeedDragon::vSetSammelrufMSN(const string & sMSN)
{
	vSetAny("99912=%s", sMSN.c_str());
}

string ocSpeedDragon::sGetSammelrufMSN(void)
{
	return sGetAny("99912");
}

void ocSpeedDragon::vSetFernprogMSN(const string & sMSN)
{
	vSetAny("99913=%s", sMSN.c_str());
}

string ocSpeedDragon::sGetFernprogMSN(void)
{
	return sGetAny("99913");
}

void ocSpeedDragon::vSetFernprogPW(const string & sPW)
{
	vSetAny("9971=%s:%s", sPW.c_str(), sPW.c_str());
}

string ocSpeedDragon::sGetFernprogPW(void)
{
	return sGetAny("9971");
}

// ohne fuehrende Nullen !!!
void ocSpeedDragon::vSetLocation(const string & sCountry, const string & sArea)
{
	vSetAny("99919=%s*%s", sCountry.c_str(), sArea.c_str());
}

string ocSpeedDragon::sGetLocation(void)
{
	return sGetAny("99919");
}

void ocSpeedDragon::vSetMSNTransmission(tMSNTransmission xType)
{
	if (bVersionGE(3, 10))
		vSetAny("99740=%d", xType);
	else
		xError = OldFirmware;
}

tMSNTransmission ocSpeedDragon::xGetMSNTransmission(void)
{
	if (bVersionGE(3, 10))
		return (tMSNTransmission)atoi(sGetAny("99740").c_str());
	else
	{
		xError = OldFirmware;
		return MSNUnknown;
	};
}

void ocSpeedDragon::vSetAnalogDienstkennung(tDienstkennung xType)
{
	if (bVersionGE(3, 10))
		vSetAny("99741=%d", xType);
	else
		xError = OldFirmware;
}

tDienstkennung ocSpeedDragon::xGetAnalogDienstkennung(void)
{
	if (bVersionGE(3, 10))
		return (tDienstkennung)atoi(sGetAny("99741").c_str());
	else
	{
		xError = OldFirmware;
		return Audio31khz;
	};
}

void ocSpeedDragon::vSetBusyTone(tPort xNo, tBusyTone xType)
{
	if (bVersionGE(3, 10))
		vSetAny("9957/%d=%d", xNo, xType);
	else
		xError = OldFirmware;
}

tBusyTone ocSpeedDragon::xGetBusyTone(tPort xNo)
{
	if (bVersionGE(3, 10))
		return (tBusyTone)atoi(sGetAny("9957/%d", xNo).c_str());
	else
	{
		xError = OldFirmware;
		return HardBusy;
	};
}

void ocSpeedDragon::vDial(tPort xNo, const string & sNumber)
{
	sprintf(cBuf, "AT&HD\"%d%s\"\r", xNo, sNumber.c_str());
	oConn.vSendString(cBuf);
	vWaitOkError();
}

tDialResult ocSpeedDragon::xWaitForDialResult(tPort xNo)
{
	char cBuf1[20];
	char cBuf2[20];
	char cBuf3[20];
	char cBuf4[20];

	sprintf(cBuf1, "DIALER %d0", xNo);
	sprintf(cBuf2, "DIALER %d1", xNo);
	sprintf(cBuf3, "DIALER %d2", xNo);
	sprintf(cBuf4, "DIALER %d3", xNo);
	
	oConn.vWaitString(cBuf1, cBuf2, cBuf3, cBuf4);
	if (oConn.iStringFound() == 0)
	{
		return NoResult;
	}
	else
	{
		return tDialResult(oConn.iStringFound());
	};
}

void ocSpeedDragon::vCallMonitorOn(void)
{
	oConn.vSendString("ATX2S107=255S108=255\r");
	vWaitOkError();
}

void ocSpeedDragon::vCallMonitorOff(void)
{
	oConn.vSendString("ATS107=4S108=0\r");
	vWaitOkError();
}

void ocSpeedDragon::vSetAny(const char * sFormat, ...)
{
	va_list xArgs;
	char * sHPFormat = (char*)malloc(strlen(sFormat)+10);

	sprintf(sHPFormat, "AT&HP\"%s\"\r", sFormat);
	va_start(xArgs, sFormat);
	vsnprintf(cBuf, SD_BUF_SIZE, sHPFormat, xArgs);
	oConn.vSendString(cBuf);
	va_end(xArgs);
	free(sHPFormat);
	vWaitOkError();
}

string ocSpeedDragon::sGetAny(const char * sFormat, ...)
{
	va_list xArgs;
	string sResult;
	// char * sHPFormat = (char*)malloc(strlen(sFormat)+10);
	// char * sWFormat = (char*)malloc(strlen(sFormat)+10);
	char sHPFormat[100];
	char sWFormat[100];
	sprintf(sHPFormat, "AT&HP\"%s?\"\r", sFormat);
	sprintf(sWFormat, "%s?:", sFormat);
	va_start(xArgs, sFormat);
	vsnprintf(cBuf, SD_BUF_SIZE, sHPFormat, xArgs);
	oConn.vSendString(cBuf);
	vsnprintf(cBuf, SD_BUF_SIZE, sWFormat, xArgs);
	oConn.vWaitString(cBuf);
	oConn.vReceiveString();
	sResult = oConn.sGetLastString();
	va_end(xArgs);
	// free(sHPFormat);
	// free(sWFormat);
	vWaitOkError();
	return sResult;
}

void ocSpeedDragon::vSetPortBool(const string & sKennziffer, tPort xNo, bool bEintrag)
{
	vSetAny("%s/%d=%d", sKennziffer.c_str(), xNo, bEintrag?1:0);
}

bool ocSpeedDragon::bGetPortBool(const string & sKennziffer, tPort xNo)
{
	return (sGetAny("%s/%d", sKennziffer.c_str(), xNo) == "1");
}

void ocSpeedDragon::vWaitOkError(void)
{
	oConn.vWaitString("OK", "ERROR", ":?");
	if (oConn.iStringFound() != 1)
	{
		xError = ATError;
		if (oConn.iStringFound() == 3)
			oConn.vWaitString("OK", "ERROR");
	};
}

bool ocSpeedDragon::bVersionGE(int iMajor, int iMinor)
{
	return ((iMajorVer*100)+iMinorVer) >= ((iMajor*100)+iMinor);
}

