//
// sd.hpp
//
// (c) Jan Willamowius, 1998
//
// free for private use - commercial use or commercial distribution
// requires permission from the author
//

#ifndef SD_HPP
#define SD_HPP

#include <string>
#include "connect.hpp"

const int SD_BUF_SIZE = 100;

typedef int tPort;
enum tAmtsBerechtigung {Halbamt=1, Ortsamt=2, National=3,  International=4};
enum tDisplayPart {	OnlyRedirectedMSN=0,
					AllMSN=1,
					CallList=2,
					Charges=3,
					BChannels=4,
					IncommingCalls=5,
					ExtendedBChannels=6};
enum tRedirectionType {CFU=0, CFNR=1, CFB=2, BK2=3};
enum tLanguage {Deutsch=0, English, Franzoesisch, Italienisch, Flaemisch, Daenisch, Norwegisch};
enum tISDNNet {ETSI=0, Belgium=1, Italy=2};
enum tDialResult {NoResult=0, Waiting=1, Done=2, Timeout=3, PortBusy=4};
enum tMSNTransmission {MSNUnknown=0, MSNNational=1};
enum tDienstkennung {Audio31khz=0, Speech=1};
enum tBusyTone {HardBusy=1, SoftBusy=2, NoBusy=3};
enum tSDError {	NoError=0,
				UnknownError,
				NotHagenuk,
				AutobaudError,
				DeviceLocked,
				OldFirmware,
				ATError};

class ocSpeedDragon
{
public:
	ocSpeedDragon(const string & sDevice, int iBps = 0);  /* Bps == 0 -> autobaud */
	virtual ~ocSpeedDragon();

	void vOpen(void);
	void vClose(void);
	bool bIsOpen(void) const;
	void vSetDebug(bool bOn);
	void vSetTimeout(const int iNewTimeout);	// seconds
	int iGetTimeout(void) const;				// seconds
	tSDError xGetLastError(void);
	// use with care: access to the low-level file descriptor
	int iGetFileDescriptor(void);

	void vSetBps(int iNewBps);	/* must be done before vOpen() */
	int iGetBps(void) const;
	void vChangeBpsPermanently(int iNewBps);	/* after vOpen(), permanent change */
	void vChangeBpsTemporary(int iNewBps);		/* after vOpen(), temporary change */

	string sGetFirmwareRevision(void) const;

	void vClearCallList(void);
	void vSetAmtsBerechtigungPort(tPort xNo, tAmtsBerechtigung xAmt);
	tAmtsBerechtigung xGetAmtsBerechtigungPort(tPort xNo);
	void vSetAmtsBerechtigungPBX(tAmtsBerechtigung xAmt);
	tAmtsBerechtigung xGetAmtsBerechtigungPBX(void);
	void vSetGebuehrenfaktorPort(tPort xNo, int iFaktor, int iNachkomma = 0);
	int iGetGebuehrenfaktorPort(tPort xNo);
	int iGetGebuehrenNachkommaPort(tPort xNo);
	void vSetGebuehrenfaktorPBX(int iFaktor, int iNachkomma = 0);
	int iGetGebuehrenfaktorPBX(void);
	int iGetGebuehrenNachkommaPBX(void);
	int iGetSummenzaehlerPort(tPort xNo);
	int iGetSummenzaehlerPBX(void);
	void vClearSummenzaehlerPort(tPort xNo);
	void vClearSummenzaehlerPBX(void);
	void vRufumleitungOn(const string & sMSN, tRedirectionType xType);
	void vRufumleitungOff(const string & sMSN, tRedirectionType xType);
	bool bGetRufumleitungActive(const string & sMSN, tRedirectionType xType);
	void vSetRufumleitung(const string & sMSN, const string & sNumber, tRedirectionType xType);
	string sGetRufumleitung(const string & sMSN, tRedirectionType xType);
	void vRufumleitungCFUOn(const string & sMSN);
	void vRufumleitungCFUOff(const string & sMSN);
	bool bGetRufumleitungCFUActive(const string & sMSN);
	void vSetRufumleitungCFU(const string & sMSN, const string & sNumber);
	string sGetRufumleitungCFU(const string & sMSN);
	void vRufumleitungCFNROn(const string & sMSN);
	void vRufumleitungCFNROff(const string & sMSN);
	bool bGetRufumleitungCFNRActive(const string & sMSN);
	void vSetRufumleitungCFNR(const string & sMSN, const string & sNumber);
	string sGetRufumleitungCFNR(const string & sMSN);
	void vRufumleitungCFBOn(const string & sMSN);
	void vRufumleitungCFBOff(const string & sMSN);
	bool bGetRufumleitungCFBActive(const string & sMSN);
	void vSetRufumleitungCFB(const string & sMSN, const string & sNumber);
	string sGetRufumleitungCFB(const string & sMSN);
	void vRufumleitungBK2On(const string & sMSN);
	void vRufumleitungBK2Off(const string & sMSN);
	bool bGetRufumleitungBK2Active(const string & sMSN);
	void vSetRufumleitungBK2(const string & sMSN, const string & sNumber);
	string sGetRufumleitungBK2(const string & sMSN);
	void vAllRufumleitungenOff(void);
	void vSetCallerIDSuppression(const string & sMSN, bool xOn);
	bool bGetCallerIDSuppression(const string & sMSN);
	void vSetSammelrufEintrag(tPort xNo, bool bOn);
	bool bGetSammelrufEintrag(tPort xNo);
	void vSetPseudoDialtone(tPort xNo, bool bOn);
	bool bGetPseudoDialtone(tPort xNo);
	void vSetAnklopfen(tPort xNo, bool bOn);
	bool bGetAnklopfen(tPort xNo);
	void vSetDisplayPart(tDisplayPart xPart, bool bOn);
	bool bGetDisplayPart(tDisplayPart xPart);
	void vSetLanguage(tLanguage xLanguage);
	tLanguage xGetLanguage(void);
	void vSetISDNNet(tISDNNet xISDN);
	tISDNNet xGetISDNNet(void);
	void vClearPortMSN(tPort xNo);
	void vClearSammelrufMSN(void);
	void vClearFernprogMSN(void);
	void vSetPortMSN(tPort xNo,const string & sMSN);
	string sGetPortMSN(tPort xNo);
	void vSetSammelrufMSN(const string & sMSN);
	string sGetSammelrufMSN(void);
	void vSetFernprogMSN(const string & sMSN);
	string sGetFernprogMSN(void);
	void vSetFernprogPW(const string & sPW);
	string sGetFernprogPW(void);
	void vSetLocation(const string & sCountry, const string & sArea);
	string sGetLocation(void);
	void vSetMSNTransmission(tMSNTransmission xType);
	tMSNTransmission xGetMSNTransmission(void);
	void vSetAnalogDienstkennung(tDienstkennung xType);
	tDienstkennung xGetAnalogDienstkennung(void);
	void vSetBusyTone(tPort xNo, tBusyTone xType);
	tBusyTone xGetBusyTone(tPort xNo);

	// dialling support
	void vDial(tPort xNo, const string & sNumber);
	tDialResult xWaitForDialResult(tPort xNo);

	// call monitor
	void vCallMonitorOn(void);
	void vCallMonitorOff(void);

protected:
	// helper functions
	void vWaitOkError(void);

	void vSetAny(const char * sFormat, ...);
	string sGetAny(const char * sFormat, ...);

	void vSetPortBool(const string & sKennziffer, tPort xNo, bool bOn);
	bool bGetPortBool(const string & sKennziffer, tPort xNo);

	// prueft, ob aktuelle Version >= geforderter Version ist
	bool bVersionGE(int iMajor, int iMinor);
	
protected:
	int iMajorVer;
	int iMinorVer;
	string sDeviceName;
	ocConnection oConn;
	int iConnBps;
	bool bDebug;
	char cBuf[SD_BUF_SIZE];	// general purpose buffer
	tSDError xError;
};

#endif
