//
// sd_monitor.cpp
//
// (c) Jan Willamowius, 1998
//
// free for private use - commercial use or commercial distribution
// requires permission from the author
//

#include <iostream>
#include "connect.hpp"
#include <time.h>
#include <signal.h>

#ifdef linux
// i made a link to the serial device my SpeedDragon is connected to
ocConnection oConn("/dev/speeddragon");
#endif
#ifdef _MSC_VER
ocConnection oConn("COM1");
#endif


#ifdef linux
void vCleanup(int iSignal)
{
	oConn.vClose();
	exit(0);
}

void vErrorCleanup(int iSignal)
{
	cerr << "Error: Signal " << iSignal << endl;
	oConn.vClose();
	exit(1);
}
#endif

int main(void)
{
	string oLine;
	int iRingPos;

#ifdef linux
	// install signal handler
	signal(SIGHUP, vCleanup);
	signal(SIGINT, vCleanup);
	signal(SIGQUIT, vCleanup);
	signal(SIGKILL, vCleanup);
	signal(SIGTERM, vCleanup);
	signal(SIGABRT, vErrorCleanup);
	signal(SIGSEGV, vErrorCleanup);
	signal(SIGIOT, vErrorCleanup);
	signal(SIGBUS, vErrorCleanup);
#endif

	cout << "SpeedDragon Call-Monitor, V0.4" << endl;
	cout << "(c) Jan Willamowius, 1998" << endl;
	cout << endl;

#ifdef linux
	oConn.vAutoSetBps();
#endif
#ifdef _MSC_VER
	oConn.vSetBps(115200);
#endif
//	oConn.vSetEcho(true);
	oConn.vOpen();
	// check for errors
	if (oConn.bIsOpen())
	{
		// Monitor an
		oConn.vSendString("ATX2S0=0S107=255S108=255\r");
		oConn.vWaitString("OK", "ERROR");
		
		for( ; ; )
		{
			oConn.vReceiveString();
			oLine = oConn.sGetLastString();
			iRingPos = oLine.find("RING", 0, 3);
			if((iRingPos >= 0) && (oLine.size() > 5))
			{
				char cBuf[100];
				time_t xNow;
				struct tm xLocal;

				// check if it's the first RING
				oConn.vSendString("ATS1?\r");
				oConn.vWaitString("ATS1?");
				oConn.vWaitString("1", "OK");

				if (oConn.iStringFound() == 1)
				{	
					xNow = time(NULL);
					xLocal = *localtime(&xNow);
					strftime(cBuf, 100, "%d.%m.%Y %H:%M", &xLocal);
					// remove line feeds etc. from oLine
					while (iRingPos-- > 0)
						oLine.erase(0,1);
					cout << cBuf << "\t" << oLine << endl;
				};
			};
		};
	}
	else
	{
		cerr << "SpeedDragon nicht ansprechbar!" << endl;
	}

	return 0;
}

