#include "detectdlg.h"

detectdlg::detectdlg(QWidget *parent, const char *name) :
	QDialog(parent, name, TRUE) 
{
	QLabel *l_bitrate, *l_device;
	QPushButton *b_ok, *b_cancel;
	
	setFixedWidth(250);
	
	layoutSetup=new QGridLayout(this, 4, 2, 10);
	
	/* device name */
	le_device=new QLineEdit(this);
	le_device->setFixedHeight(25);
	layoutSetup->addWidget(le_device, 1, 1);
	l_device=new QLabel(le_device,"&Gertenamen:", this);
	l_device->setFixedHeight(25);
	layoutSetup->addWidget(l_device, 1, 0);
	
	/* bitrate */
	combo_bitrate=new QComboBox(FALSE, this);
	combo_bitrate->setStyle(WindowsStyle);
	combo_bitrate->setFixedHeight(25);
	connect(combo_bitrate, SIGNAL(activated(int)), SLOT(bpsChanged()));
	layoutSetup->addWidget(combo_bitrate, 2,1);
	l_bitrate=new QLabel(combo_bitrate,"&Bitrate:", this);
	l_bitrate->setFixedHeight(25);
	layoutSetup->addWidget(l_bitrate, 2, 0);
	
	/* buttons */
	b_ok=new QPushButton("Ok", this);
	layoutSetup->addWidget(b_ok, 3, 0);
	b_ok->setFixedSize(100, 30);
	b_ok->setDefault(TRUE);
	connect(b_ok, SIGNAL(clicked()), SLOT(accept()));
	connect(b_ok, SIGNAL(clicked()), SIGNAL(setupChanged()));
	b_cancel=new QPushButton("Abbbrechen", this);
	layoutSetup->addWidget(b_cancel, 3, 1);
	b_cancel->setFixedSize(100, 30);
	connect(b_cancel, SIGNAL(clicked()), SLOT(reject()));
	
	resetStatusChanged();
}

void detectdlg::setupWin(int errno, int bps, const char *deviceName) {
	
	QLabel *l_errText;
	int index;
	int ibitrates[]={ 9600, 19200, 38400, 57600, 115200, 230400 };
	const char *bitrates[]={ "9600", "19200", "38400", "57600", "115200", "230400", 0 };
	char *error[4];
	error[0]="Hinweis:\nStellen Sie die Bitrate nicht auf einen, von Ihrer\
		Hardware nicht untersttzen Wert ein!";
	error[1]="Die angegebene Gertedatei ist nicht vorhanden.\
		Bitte berprfen Sie den Gertenamen."; 
	error[2]="Das angegebene Gert ist kein zeichenorientiertes Gert.\
		Bitte berprfen Sie den Gertenamen.";
	error[3]="Die Bitrate des Terminal-Adapters konnte\
		nicht ermittelt werden. Bitte berprfen Sie den Gertenamen\
		und den korrekten Anschluss des Gertes.";
	
	l_errText=new QLabel(this);
	l_errText->setAlignment(WordBreak | AlignVCenter);
	l_errText->setAutoResize(TRUE);
	l_errText->setFixedHeight(80);
	l_errText->setText(error[errno]);
	layoutSetup->addMultiCellWidget(l_errText, 0, 0, 0, 1);
	if (errno==0) { /* selected adapter setup from menu */
		setCaption("Anlagen - Setup");
		combo_bitrate->clear();	/* clear combo list */
		combo_bitrate->insertStrList(bitrates); 
		combo_bitrate->setEnabled(TRUE);
		le_device->setEnabled(FALSE);	/* device can't be changed */
		for (index=0; index<=5; index++)  /* sets current item */
			if (bps==ibitrates[index])
				combo_bitrate->setCurrentItem(index);
	}
	else {	
		setCaption("Autodetection");
		combo_bitrate->clear();
		combo_bitrate->insertItem("Auto"); /* only auto is in list */
		combo_bitrate->setEnabled(FALSE);
		le_device->setEnabled(TRUE);
	}
	le_device->setText(deviceName);
	le_device->setMaxLength(MAX_DEVICE_NAME);
	qApp->processEvents(); /* this completes the resize event before show() */
}

int detectdlg::getBps(void) {
	int ibitrates[]={ 9600, 19200, 38400, 57600, 115200, 230400 };
	if (combo_bitrate->count()==1)
		return(0); /* Autodetect */
	else
		return(ibitrates[combo_bitrate->currentItem()]);
}

void detectdlg::resetStatusChanged(void) {
	iBpsChanged=FALSE;
}
