#include "divdlg.h"

divDlg::divDlg(QWidget *parent, const char *name) : QDialog(parent, name, TRUE)
{
	setFixedSize(500, 220);
	setCaption("Rufumleitungen");
	
	divlist=new QListView(this);
	divlist->setGeometry(10,10, width()-20, height()-60);	
	divlist->setFrameStyle(QFrame::WinPanel | QFrame:: Sunken);
	divlist->setAllColumnsShowFocus(TRUE);
	divlist->addColumn("MSN", 100);
	divlist->addColumn("Port", 100);
	divlist->addColumn("Umleitung", 100);
	divlist->addColumn("Rufziel", 176);
	connect(divlist, SIGNAL(selectionChanged(QListViewItem*)), SLOT(MSNSelected(QListViewItem*)));
	connect(divlist, SIGNAL(doubleClicked(QListViewItem*)), SLOT(activationDialog()));
	
	b_act=new QPushButton("Aktivieren", this);
	b_act->setGeometry(10, 182, 90, 25);
	b_act->setEnabled(FALSE);
	connect(b_act, SIGNAL(clicked()), SLOT(activationDialog()));
	b_deact=new QPushButton("Deaktivieren", this);
	b_deact->setGeometry(120, 182, 90, 25);
	b_deact->setEnabled(FALSE);
	connect(b_deact, SIGNAL(clicked()), SIGNAL(deactivate()));
	b_all=new QPushButton("Alle deaktivieren", this);
	b_all->setGeometry(230, 182, 120, 25);
	b_all->setEnabled(FALSE);
	connect(b_all, SIGNAL(clicked()), SIGNAL(deactivateAll()));
	QPushButton *b_ok=new QPushButton("Ok", this);
	b_ok->setGeometry(390, 182, 100, 25);
	b_ok->setDefault(TRUE);
	connect(b_ok, SIGNAL(clicked()), SLOT(accept()));
}

/* adds an entry to the diversions list */
void divDlg::addMSN(const char *MSN, divPort port, divStatus status, 
	const char* divID)
{
	QListViewItem *item;
	char str_port[20];
	char str_status[20];

	switch (port) {
		case Port1: 
		case Port2:
		case Port3:
			sprintf(str_port, "Port %i", (int)port+1);
			break;
		case PC:
			strcpy(str_port, "PC");
			break;
		case Collective:
			strcpy(str_port, "Sammelruf");
	}
	
	switch (status) {
		case divOff:
			strcpy(str_status,"-");
			break;
		case divBK2:
			strcpy(str_status, "B-Kanal");
			break;
		case divCFU:
			strcpy(str_status, "Amt (CFU)");
			break;
		case divCFB:
			strcpy(str_status, "Amt (CFB)");
			break;
		case divCFNR:
			strcpy(str_status, "Amt (CFNR)");
			break;
		case divCFNRCFB:
			strcpy(str_status, "Amt (CFNR/CFB)");
			break;
	}	
	item=new QListViewItem(divlist, MSN, str_port, str_status, divID);
	if (divlist->firstChild()!=0) /* enables button when first item is added */
		b_all->setEnabled(TRUE);
}

/* removes all entries of the diversions list */
void divDlg::removeAllMSNs(void) {
	divlist->clear();
	b_act->setEnabled(FALSE);
	b_deact->setEnabled(FALSE);
	b_all->setEnabled(FALSE);
}

/* returns MSN of currently selected list item */
const char* divDlg::getSelectedMSN(void) {
	return(divlist->currentItem()->text(0));
}

/* returns diversion type of currently selected list item */
divStatus divDlg::getSelectedType(void) {
	if (strcmp(divlist->currentItem()->text(2), "Amt (CFU)")==0)
		return(divCFU);
	else if (strcmp(divlist->currentItem()->text(2), "Amt (CFB)")==0)
		return(divCFB);
	else if (strcmp(divlist->currentItem()->text(2), "Amt (CFNR)")==0)
		return(divCFNR);
	else if (strcmp(divlist->currentItem()->text(2), "Amt (CFNR/CFB)")==0)
		return(divCFNRCFB);
	else if (strcmp(divlist->currentItem()->text(2), "B-Kanal")==0)
		return(divBK2);
	else
		return(divOff);
}

const char* divDlg::getSelectedID(int position=0) {

	static char ID[MAX_ID];
	int i;

	if (getSelectedType()==divCFNRCFB) {
		strcpy(ID, "\0");
		for (i=0; i<(int)strlen(divlist->currentItem()->text(3)); i++) {
			if ((position==0) && (divlist->currentItem()->text(3)[i]=='/')) {
				break;
			}
			strncat(ID, &(divlist->currentItem()->text(3)[i]), 1);
			if ((position==1) && (divlist->currentItem()->text(3)[i]=='/')) {
				strcpy(ID, "\0");
			}
		}
		return(ID);
	}			
	return(divlist->currentItem()->text(3));
}

/* enables/disables dialog buttons according to state of the selected MSN */
void divDlg::MSNSelected(QListViewItem *item) {
	if(strcmp(item->text(2),"-")==0) {
		b_act->setEnabled(TRUE);
		b_act->setText("Aktivieren");
		b_deact->setEnabled(FALSE);
	}
	else {
		b_act->setEnabled(TRUE);
		b_act->setText("Bearbeiten");
		b_deact->setEnabled(TRUE);
	}	
}

void divDlg::activationDialog(void) {
	
	QDialog *actDlg=new QDialog(this, 0, TRUE);
	char caption[256];
	char destination[256];
	
	sprintf(caption,"Rufumleitung fr MSN %s (%s)",  
		divlist->currentItem()->text(0), divlist->currentItem()->text(1));
	actDlg->setCaption(caption);
	actDlg->setFixedSize(300, 235);
	
	/* buttons */
	b_CFOK=new QPushButton("Ok", actDlg);
	b_CFOK->setGeometry(110, 205, 80, 25);
	b_CFOK->setDefault(TRUE);
	connect(b_CFOK, SIGNAL(clicked()), actDlg, SLOT(accept()));
	QButton *b_cancel=new QPushButton("Abbruch", actDlg);
	b_cancel->setGeometry(210, 205, 80, 25);
	connect(b_cancel, SIGNAL(clicked()), actDlg, SLOT(reject()));
	
	/* diverson BK2 */
	QButtonGroup *g_type=new QButtonGroup("ber zweiten B-Kanal...", actDlg);
	g_type->setGeometry(10, 10, 280, 50);
	cb_BK2=new QCheckBox("sofort (CFBK2)", g_type);
	cb_BK2->move(10,20);
	cb_BK2->adjustSize();
	g_type->insert(cb_BK2);
	le_IDBK2=new QLineEdit(g_type);
	le_IDBK2->setGeometry(150, 20, 100, 20);
	le_IDBK2->setMaxLength(MAX_ID);
		
	connect(g_type, SIGNAL(clicked(int)), SLOT(CFClicked()));
	
	/* network diversions */
	QButtonGroup *g_addType=new QButtonGroup("ber Amt...", actDlg);
	g_addType->setGeometry(10, 80, 280, 110);
	cb_CFU=new QCheckBox("sofort (CFU)", g_addType);
	cb_CFU->move(10,15);
	g_addType->insert(cb_CFU);
	le_IDCFU=new QLineEdit(g_addType);
	le_IDCFU->setGeometry(150, 20, 100, 20);
	le_IDCFU->setMaxLength(MAX_ID);
	
	cb_CFNR=new QCheckBox("verzgert (CFNR)", g_addType);
	cb_CFNR->move(10,50);
	cb_CFNR->adjustSize();
	g_addType->insert(cb_CFNR);
	le_IDCFNR=new QLineEdit(g_addType);
	le_IDCFNR->setGeometry(150, 50, 100, 20);
	le_IDCFNR->setMaxLength(MAX_ID);
	
	cb_CFB=new QCheckBox("bei Besetzt (CFB)", g_addType);
	cb_CFB->move(10,80);
	g_addType->insert(cb_CFB);
	cb_CFB->adjustSize();
	le_IDCFB=new QLineEdit(g_addType);
	le_IDCFB->setGeometry(150, 80, 100, 20);
	le_IDCFB->setMaxLength(MAX_ID);
	
	connect(g_addType, SIGNAL(clicked(int)), SLOT(CFClicked()));
	
	/* if MSN is already diverted get values from list view */
	if (getSelectedType() != divOff) {
		if(getSelectedType()==divCFU) {
			cb_CFU->setChecked(TRUE);
			le_IDCFU->setText(getSelectedID());
		}
		else if(getSelectedType()==divCFNR) {
			cb_CFNR->setChecked(TRUE);
			le_IDCFNR->setText(getSelectedID());
		}
		else if(getSelectedType()==divCFB) {
			cb_CFB->setChecked(TRUE);
			le_IDCFB->setText(getSelectedID());
		}
		else if(getSelectedType()==divCFNRCFB) {
			cb_CFNR->setChecked(TRUE);
			cb_CFB->setChecked(TRUE);
			le_IDCFNR->setText(getSelectedID(0));
			le_IDCFB->setText(getSelectedID(1));
		}
		else {
			le_IDBK2->setText(getSelectedID());
			cb_BK2->setChecked(TRUE);
		}
	}
	
	/* enable/disable widgets according to selection */
	CFClicked();
	
	if (actDlg->exec()) {
		if (cb_CFU->isChecked() || cb_CFB->isChecked() || cb_CFNR->isChecked()) {
			if (cb_CFU->isChecked()) {
				divlist->currentItem()->setText(3, le_IDCFU->text());
				divlist->currentItem()->setText(2, "Amt (CFU)");
			}
			if (cb_CFNR->isChecked() && cb_CFB->isChecked()) {
				sprintf(destination, "%s/%s", le_IDCFNR->text(), le_IDCFB->text());
				divlist->currentItem()->setText(3, destination);
				divlist->currentItem()->setText(2, "Amt (CFNR/CFB)");
			}
			else {
				if (cb_CFNR->isChecked()) {
					divlist->currentItem()->setText(3, le_IDCFNR->text());
					divlist->currentItem()->setText(2, "Amt (CFNR)");
				}
				if (cb_CFB->isChecked()) {
					divlist->currentItem()->setText(3, le_IDCFB->text());
					divlist->currentItem()->setText(2, "Amt (CFB)");
				}
			}
		}
		else {
			divlist->currentItem()->setText(2, "B-Kanal");
			divlist->currentItem()->setText(3, le_IDBK2->text());
		}
		emit activate();
	}
	
	delete cb_CFU;
	delete cb_CFB;
	delete cb_CFNR;
	delete cb_BK2;
	delete actDlg;
}

void divDlg::CFClicked(void) {
	if (cb_BK2->isChecked()) {
		cb_CFU->setEnabled(FALSE);
		cb_CFB->setEnabled(FALSE);
		cb_CFNR->setEnabled(FALSE);
		le_IDCFU->setEnabled(FALSE);
		le_IDCFB->setEnabled(FALSE);
		le_IDCFNR->setEnabled(FALSE);
		b_CFOK->setEnabled(TRUE);
	}
	else if (cb_CFU->isChecked()) {
		cb_CFNR->setEnabled(FALSE);
		cb_CFB->setEnabled(FALSE);
		cb_BK2->setEnabled(FALSE);
		le_IDBK2->setEnabled(FALSE);
		le_IDCFB->setEnabled(FALSE);
		le_IDCFNR->setEnabled(FALSE);
		b_CFOK->setEnabled(TRUE);
	}
	else if (cb_CFNR->isChecked() || cb_CFB->isChecked()) {
		cb_BK2->setEnabled(FALSE);
		cb_CFU->setEnabled(FALSE);
		le_IDBK2->setEnabled(FALSE);
		le_IDCFU->setEnabled(FALSE);
		b_CFOK->setEnabled(TRUE);
	}
	else {
		cb_CFU->setEnabled(TRUE);
		cb_CFNR->setEnabled(TRUE);
		cb_CFB->setEnabled(TRUE);
		cb_BK2->setEnabled(TRUE);
		le_IDCFU->setEnabled(TRUE);
		le_IDCFB->setEnabled(TRUE);
		le_IDCFNR->setEnabled(TRUE);
		le_IDBK2->setEnabled(TRUE);
		b_CFOK->setEnabled(FALSE);
	}
}
