#include "netdlg.h"

#ifdef ENGLISH
static const char *netISDNNet[]={ "ETSI", "Belgium", "Italy", 0 };
static const char *netTrans[]={ "national", "unknown", 0 };
static const char *netService[]={ "3.1 kHz audio", "speech", 0 };
static const char *netCaption="General configuration";
static const char *netLabelISDNNet="ISDN network:";
static const char *netLabelTrans="MSN transmission:";
static const char *netLabelService="Service ID:";
static const char *netOkButton="Ok";
static const char *netCancelButton="Cancel";
#else
static const char *netISDNNet[]={ "ETSI", "Belgien", "Italien", 0 };
static const char *netTrans[]={ "national", "unbekannt", 0 };
static const char *netService[]={ "3.1 kHz Audio", "Sprache", 0 };
static const char *netCaption="Allgemeine Konfiguration";
static const char *netLabelISDNNet="Netzkennung:";
static const char *netLabelTrans="MSN-bermittlung:";
static const char *netLabelService="Dienstekennung:";
static const char *netOkButton="Ok";
static const char *netCancelButton="Abbruch";
#endif

netDlg::netDlg(QWidget *parent, const char *name) 
	: QDialog(parent, name, TRUE)
{
	QHBoxLayout base(this);
	
	setFixedSize(380,120);
	setCaption(netCaption);
	
	/*left margin */
	base.addSpacing(10);
	
	/* labels */
	QBoxLayout *buttons=new QVBoxLayout();
	base.addLayout(buttons, 2);
	
	buttons->addSpacing(15);
	QLabel *l_net=new QLabel(netLabelISDNNet, this);
	buttons->addWidget(l_net);
	buttons->addSpacing(10);
	
	QLabel *l_trans=new QLabel(netLabelTrans, this);
	buttons->addWidget(l_trans);
	buttons->addSpacing(10);
	
	QLabel *l_service=new QLabel(netLabelService, this);
	buttons->addWidget(l_service);
	buttons->addSpacing(15);
		
	/* combos */
	buttons=new QVBoxLayout();
	base.addLayout(buttons, 2);
	
	buttons->addSpacing(15);
	combo_net=new QComboBox(FALSE, this);
	combo_net->insertStrList(netISDNNet);
	connect(combo_net, SIGNAL(activated(int)), SLOT(ISDNNetChanged()));
	buttons->addWidget(combo_net);
	buttons->addSpacing(10);
	
	combo_trans=new QComboBox(FALSE, this);
	combo_trans->insertStrList(netTrans);
	connect(combo_trans, SIGNAL(activated(int)), SLOT(MSNTransChanged()));
	buttons->addWidget(combo_trans);
	buttons->addSpacing(10);
	
	combo_service=new QComboBox(FALSE, this);
	combo_service->insertStrList(netService);
	connect(combo_service, SIGNAL(activated(int)), SLOT(serviceChanged()));
	buttons->addWidget(combo_service);
	buttons->addSpacing(15);
	
	base.addStretch(1);
	
	/* buttons */
	buttons=new QVBoxLayout();
	base.addLayout(buttons, 1);
	buttons->addSpacing(30);
	QPushButton *b_ok=new QPushButton(netOkButton, this);
	b_ok->setDefault(TRUE);
	buttons->addWidget(b_ok);
	buttons->addSpacing(10);
	QPushButton *b_cancel=new QPushButton(netCancelButton, this);
	buttons->addWidget(b_cancel);
	buttons->addSpacing(30);
	
	/* right margin */
	base.addSpacing(10);
	
	base.activate();
	setFixedSize(size());
	
	connect(b_ok, SIGNAL(clicked()), SLOT(accept()));
	connect(b_cancel, SIGNAL(clicked()), SLOT(reject()));
	
	resetStatusChanged();
}

void netDlg::setISDNNet(tISDNNet type) {
	combo_net->setCurrentItem(type);
}

tISDNNet netDlg::getISDNNet(void) {
	return((tISDNNet)combo_net->currentItem());
}

void netDlg::setMSNTrans(tMSNTransmission type) {
	combo_trans->setCurrentItem(type);
}

tMSNTransmission netDlg::getMSNTrans(void) {
	return((tMSNTransmission)combo_trans->currentItem());
}

void netDlg::setService(tDienstkennung type) {
	combo_service->setCurrentItem(type);
}

tDienstkennung netDlg::getService(void) {
	return((tDienstkennung)combo_service->currentItem());
}

void netDlg::resetStatusChanged(void) {
	Changed.MSNTrans=FALSE;
	Changed.ISDNNet=FALSE;
	Changed.service=FALSE;
}
