#include "qtsd.h"

#include "fileopen.xpm"
#include "filesave.xpm"
#include "fileprint.xpm"
#include "help.xpm"
#include "sdwrite.xpm"
#include "sdread.xpm"
#include "diversion.xpm"

static const char *sModemLocked="Die angegebene Gertedatei wird von\
	einem\nanderen Programm benutzt. Schlieen Sie\ndieses Programm um\
	fortzufahren.\n";
static const char* sOldFirmware="Die Firmware des Speed Dragon ist zu\
	\nalt. Eine neuere Versionen gibt es unter\
	\nhttp://www.hagenuk.de";
static const char* sUnknownError="Ein unbekannter Fehler ist aufgetreten.\
	\nBitte benachrichtigen Sie den Autor.";

QTSD::QTSD(QWidget *parent, const char *name) : QMainWindow(parent,name) {	
	
	setCaption("QtDragon");
	setDeviceName("/dev/modem");
	strcpy(configFile, "\0");
	
	/* menubar */
	menubar=menuBar(); 
	createMenubar(); 
	
	/* toolbar */
	createToolbar();
					    
	/* statusbar */
	statusbar=statusBar();
	connect(this, SIGNAL(readingConfig(const char*)), SLOT(showStatus(const char*)));
	connect(this, SIGNAL(writingConfig(const char*)), SLOT(showStatus(const char*)));
	connect(this, SIGNAL(doneConfig(const char*)), SLOT(showStatus(const char*)));
	
	/* central widget (tabdialog) */
	QFrame *tmp=new QFrame(this);
	tmp->setFrameStyle(QFrame::WinPanel | QFrame::Sunken);
	tabdialog=new tabDialog(tmp);
	tabdialog->move(5,8);
	createTabdialog();
	 
	setCentralWidget(tmp);
	connect(tabdialog, SIGNAL(tabSelected(int)), SLOT(tabMSNSelected(int)));
	setFixedSize(5+tabdialog->width()+5, 80+tabdialog->height()+8);
	readRcFile(); /* parses qtdragonrc */
}

QTSD::~QTSD(void) {

	/* write configuration to rcFile *
	 *********************************/
	rcFile configFile(".qtdragonrc", getenv("HOME"));
	#ifdef 	_DEBUG
	printf("\nparsing rc-file:\n");
	#endif
	configFile.readRcFile();
	/* toolbar*/
	if (toolbar->isVisible()) {
		configFile.setKey("view", "toolbar", "true");
	}
	else {
		configFile.setKey("view", "toolbar", "false");
	}
	/* statusbar */
	if (statusbar->isVisible()) {
		configFile.setKey("view", "statusbar", "true");
	}
	else {
		configFile.setKey("view", "statusbar", "false");
	}
	/* devicename */
	configFile.setKey("device", "devicename", getDeviceName());
	
	configFile.purgeRcFile(false, false, true);
	configFile.writeRcFile();
	
	/* close com-port   *
	*********************/
	SD->vClose();
}

void QTSD::initSpeedDragon(int bps) {
	
	bool Failed=FALSE;
	struct stat dev;
	/* using tabdialog as parent - so the window will stay unresizeable */
	/* just another curious thing! */
	detectdlg *ddialog=new detectdlg(tabdialog); 
	
	setFixedSize(width(), height()); 
	userEventEnabled(FALSE);
	emit readingConfig("Detecting Speed Dragon...");
	if (stat(getDeviceName(), &dev)==-1) {
		ddialog->setupWin(1, 0, getDeviceName());
		Failed=TRUE;
	}
	else if (!S_ISCHR(dev.st_mode)) {
		ddialog->setupWin(2, 0, getDeviceName());
		Failed=TRUE;
	}
	else {	
		SD=new ocSpeedDragon(getDeviceName(), bps);
		SD->vOpen();	
		switch (SD->xGetLastError()) { /* check if error occured */
			case NoError:
				break;
			case OldFirmware:
				userEventEnabled(TRUE);
				QMessageBox::critical(tabdialog, "Firmware", 
						sOldFirmware, "Ok", 0);
				exit(1);
				break;
			case UnknownError:
				userEventEnabled(TRUE);
				QMessageBox::critical(tabdialog, "Unbekannter Fehler", 
						sUnknownError, "Ok", 0);
				exit(1);
				break;
			default:
				userEventEnabled(TRUE);
				delete SD;
				ddialog->setupWin(3, 0, getDeviceName()); /* set Text and Widgets */
				Failed=TRUE;
				break;
			case DeviceLocked:
				userEventEnabled(TRUE);
				do {
					switch(QMessageBox::critical(tabdialog, getDeviceName(), 
						sModemLocked, "Ok", "Abbruch", 0, 1)) {
						case 0:
							SD->vOpen();
							break;
						case 1:
							exit(1);
					}
				} while(SD->xGetLastError()==DeviceLocked);
				break;
		}
	}
	if (Failed) {
		userEventEnabled(TRUE);
		if (ddialog->exec()) { /* if ok pressed */
			setDeviceName(ddialog->getDeviceName());
			initSpeedDragon(0);
			delete ddialog;				
		}
		else { /* if cancel pressed */
			delete ddialog;
			exit(1); 
		}
	}	
	else {
		readConfiguration();
	}
	userEventEnabled(TRUE);
}

void QTSD::readRcFile(void) { 
	
	const char *item;
	
	rcFile configFile(".qtdragonrc", getenv("HOME"));
	#ifdef 	_DEBUG
	printf("\nparsing rc-file:\n");
	#endif
	configFile.readRcFile();
	/* get toolbar status */
	if ((item=configFile.getKey("view", "toolbar")))
		if (strcmp(item, "false")==0)
			toolbarToggled();
	/* get statusbar status */
	if ((item=configFile.getKey("view", "statusbar"))) 
		if (strcmp(item, "false")==0)
			statusbarToggled();
	/* get default device name */
	if ((item=configFile.getKey("device", "devicename"))) 
			setDeviceName(item);
}

void QTSD::readConfiguration(void) {
	
	string str;
	int country_code, i;
	char area[20], a;
	
	userEventEnabled(FALSE);
	
	/*Adapter*/
	emit readingConfig("Lese Konfiguration: Anlage...");
	adapter->setAuthorization(SD->xGetAmtsBerechtigungPBX());
	adapter->setCounter(SD->iGetSummenzaehlerPBX());
	adapter->setFactor(SD->iGetGebuehrenfaktorPBX(), 
		SD->iGetGebuehrenNachkommaPBX());
	str=SD->sGetLocation();
	sscanf(str.c_str(), "%i%c%s", &country_code, &a, area);
	adapter->setCountry(country_code);
	adapter->setAreaCode(area);
	
	/*Display*/
	emit readingConfig("Lese Konfiguration: Display...");
	display->setLanguage(SD->xGetLanguage());
	if (SD->bGetDisplayPart(OnlyRedirectedMSN))
		display->setMSN(MSN_Diverted);
	else if (SD->bGetDisplayPart(AllMSN))
		display->setMSN(MSN_All);
	else display->setMSN(MSN_Off);
	
	if (SD->bGetDisplayPart(ExtendedBChannels))
		display->setBChannel(B_Advanced);
	else if (SD->bGetDisplayPart(BChannels))
		display->setBChannel(B_Normal);
	else display->setBChannel(B_Off);
	
	display->setCharge(SD->bGetDisplayPart(Charges));
	display->setList(SD->bGetDisplayPart(CallList));
	display->setCaller(SD->bGetDisplayPart(IncommingCalls));
	
	/*Ports*/
	for (i=1; i<=4; i++) {
		char message[15];
	sprintf(message,"Lese Konfiguration: Port %i...", i);
	emit readingConfig(message);
		port[i]->setAuthorization(SD->xGetAmtsBerechtigungPort(i));
		str=SD->sGetPortMSN(i);
		port[i]->setMSN(str.c_str());
		port[i]->setCounter(SD->iGetSummenzaehlerPort(i));
		port[i]->setFactor(SD->iGetGebuehrenfaktorPort(i), 
			SD->iGetGebuehrenNachkommaPort(i));
		if (i!=4) {
			port[i]->setCallWaiting(SD->bGetAnklopfen(i));
			port[i]->setDialTone(SD->bGetPseudoDialtone(i));
			port[i]->setBusyTone(SD->xGetBusyTone(i));
		}
		if (strlen(port[i]->getMSN()) > 0) {
			port[i]->setMSNSuppression(SD->bGetCallerIDSuppression(port[i]->getMSN()));
		}
		else {
			port[i]->setMSNSuppression(FALSE);
		}
	}
	
	/* Collective Call */
	emit readingConfig("Lese Konfiguration: Sammelruf...");
	str=SD->sGetSammelrufMSN();
	collective->setMSN(str.c_str());
	for (i=1; i<=4; i++)
		collective->setPort(i, SD->bGetSammelrufEintrag(i));
	if (strlen(collective->getMSN()) > 0) {
			collective->setMSNSuppression(SD->bGetCallerIDSuppression(collective->getMSN()));
		}
	else {
		collective->setMSNSuppression(FALSE);
	}
		
	resetStatusChanged();
	tabMSNSelected(7); /* updates content of tabMSN */
	emit doneConfig("Bereit.");
	userEventEnabled(TRUE);
}

void QTSD::writeConfiguration(void) {
	
	char str[20];
	int i=1;
	
	userEventEnabled(FALSE);
	
	/*Adapter*/
	emit writingConfig("Schreibe Konfiguration: Anlage...");
	if (adapter->isAuthorizationChanged()) {
		#ifdef _DEBUG
		printf("writing authorization/adapter...\n");
		#endif
		SD->vSetAmtsBerechtigungPBX(adapter->getAuthorization());
	}
	if (adapter->isFactorChanged()) {
		#ifdef _DEBUG
		printf("writing factor/adapter...\n");
		#endif
		SD->vSetGebuehrenfaktorPBX(adapter->getFactor(), 
			adapter->getNachkommaFactor());
	}
	if (adapter->isCounterChanged()) {
		#ifdef _DEBUG
		printf("resetting counter/adapter...\n");
		#endif
		SD->vClearSummenzaehlerPBX();
	}	
	if (adapter->isAreaCodeChanged()) {
		sprintf(str,"%i",adapter->getCountry());
		#ifdef _DEBUG
		printf("writing area code/adapter...\n");
		#endif
		SD->vSetLocation(str, adapter->getAreaCode());
	}
	
	/*Display*/
	emit writingConfig("Schreibe Konfiguration: Display...");
	if (display->isLanguageChanged()) {
		#ifdef _DEBUG
		printf("writing language/display...\n");
		#endif
		SD->vSetLanguage(display->getLanguage());
	}
	if (display->isMSNChanged()) {
		#ifdef _DEBUG
		printf("writing MSN/display...\n");
		#endif
		switch (display->getMSN()) {
			case MSN_Diverted:
				SD->vSetDisplayPart(OnlyRedirectedMSN, TRUE);
				break;
			case MSN_All:
				SD->vSetDisplayPart(AllMSN, TRUE);
				break;
			case MSN_Off:
				SD->vSetDisplayPart(OnlyRedirectedMSN, FALSE);
				SD->vSetDisplayPart(AllMSN, FALSE);
		}
	}
	if (display->isBChannelChanged()) {
		#ifdef _DEBUG
		printf("writing b-channel/display...\n");
		#endif
		switch (display->getBChannel()) {
			case B_Normal:
				SD->vSetDisplayPart(BChannels, TRUE);
				break;
			case B_Advanced:
				SD->vSetDisplayPart(ExtendedBChannels, TRUE);
				break;
			case B_Off:
				SD->vSetDisplayPart(BChannels, FALSE);
				SD->vSetDisplayPart(ExtendedBChannels, FALSE);
		}
	}
	if (display->isChargeChanged()) {
		#ifdef _DEBUG
		printf("writing charge info/display...\n");
		#endif
		SD->vSetDisplayPart(Charges, display->getCharge());
	}
	if (display->isCallerChanged()) {
		#ifdef _DEBUG
		printf("writing caller id/display...\n");
		#endif
		SD->vSetDisplayPart(IncommingCalls, display->getCaller());
	}
	if (display->isListChanged()) {
		#ifdef _DEBUG
		printf("writing call list/display...\n");
		#endif
		SD->vSetDisplayPart(CallList, display->getList());
	}
	
	/* first clear all changed MSNs to avoid to writing
	   duplicate MSNs */
	for (i=1; i<=4; i++) {
		if (port[i]->isMSNChanged()) {
			SD->vClearPortMSN(i);
			#ifdef _DEBUG
			printf("clearing MSN/port %i...\n",i);
			#endif
		}
	}
	if (collective->isMSNChanged()==TRUE) {
		#ifdef _DEBUG
		printf("clearing MSN/collective call...\n");
		#endif
		SD->vClearSammelrufMSN();
	}
	
	/*Ports*/
	for (i=1; i<=4; i++) {
		emit writingConfig("Schreibe Konfiguration: Ports...");
		if (port[i]->isAuthorizationChanged()) {
			SD->vSetAmtsBerechtigungPort(i, port[i]->getAuthorization());
			#ifdef _DEBUG
			printf("writing authorization/port %i...\n",i);
			#endif
		}
		if (port[i]->isFactorChanged()) {
			printf("Wert: %i\n",port[i]->getFactor());
			SD->vSetGebuehrenfaktorPort(i,port[i]->getFactor(), 
				port[i]->getNachkommaFactor());
			#ifdef _DEBUG
			printf("writing factor/port %i...\n",i);
			
			#endif
		}
		if (port[i]->isMSNChanged()) {
			if (strlen(port[i]->getMSN())!=0) {
				SD->vSetPortMSN(i, port[i]->getMSN());
				#ifdef _DEBUG
				printf("writing MSN/port %i...\n",i);
				#endif
			}
			else {
				SD->vClearPortMSN(i);
				#ifdef _DEBUG
				printf("clearing MSN/port %i...\n",i);
				#endif
			}
		}
		if (port[i]->isCounterChanged()) {
			SD->vClearSummenzaehlerPort(i);
			#ifdef _DEBUG
			printf("resetting counter/port %i...\n",i);
			#endif
		}
		if (port[i]->isMSNSuppressionChanged()) {
			SD->vSetCallerIDSuppression(port[i]->getMSN(), port[i]->getMSNSuppression());
			#ifdef _DEBUG
			printf("writing suppression/port %i...\n",i);
			#endif
		}
		if (i!=4) {
			if (port[i]->isCallWaitingChanged()) {
				SD->vSetAnklopfen(i, port[i]->getCallWaiting());
				#ifdef _DEBUG
				printf("writing call waiting/port %i...\n",i);
				#endif
			}
			if (port[i]->isDialToneChanged()) {
				SD->vSetPseudoDialtone(i, port[i]->getDialTone());
				#ifdef _DEBUG
				printf("writing dial tone/port %i...\n",i);
				#endif
			}
			if (port[i]->isBusyToneChanged()) {
				SD->vSetBusyTone(i, port[i]->getBusyTone());
				#ifdef _DEBUG
				printf("writing busy tone/port %i...\n",i);
				#endif
			}
		}
	}
	
	/* Collective Call */
	if (collective->isMSNChanged()==TRUE) {
		if (strlen(collective->getMSN())==0) {
			#ifdef _DEBUG
			printf("clearing MSN/collective call...\n");
			#endif
			SD->vClearSammelrufMSN();
		}
		else
			#ifdef _DEBUG
			printf("writing MSN/collective call...\n");
			#endif
			SD->vSetSammelrufMSN(collective->getMSN());
	}
	for (i=1; i<=4; i++) {
		if (collective->isPortChanged(i)==TRUE) {
			#ifdef _DEBUG
			printf("writing port %i/collective call...\n", i);
			#endif
			SD->vSetSammelrufEintrag(i, collective->getPort(i));
		}
	}
	if (collective->isMSNSuppressionChanged()) {
		SD->vSetCallerIDSuppression(collective->getMSN(), collective->getMSNSuppression());
		#ifdef _DEBUG
		printf("writing suppression/collective call...\n");
		#endif
	}
	
	emit doneConfig("Bereit.");
	readConfiguration();
	userEventEnabled(TRUE);
}

void QTSD::saveConfiguration(void) {

	char key[20];
	char portname[10];
	int i;
	
	if (strlen(configFile)==0) { /* no save file specified yet */
		saveAsConfiguration();
		return;
	}
	
	rcFile saveConf(configFile);
	
	/* general */
	saveConf.setKey("general", "version", "0.7.3");
	
	/* adapter */
	sprintf(key, "%i", (int)adapter->getAuthorization()); 
	saveConf.setKey("adapter", "authorization", key);
	sprintf(key, "%i", (int)adapter->getCountry());
	saveConf.setKey("adapter", "country", key);
	saveConf.setKey("adapter", "area code", adapter->getAreaCode());
	sprintf(key, "%i", (int)adapter->getFactor());
	saveConf.setKey("adapter", "factor", key);
	
	/* display */
	sprintf(key, "%i", (int)display->getLanguage()); 
	saveConf.setKey("display", "language", key);
	sprintf(key, "%i", (int)display->getMSN()); 
	saveConf.setKey("display", "MSN", key);
	sprintf(key, "%i", (int)display->getBChannel()); 
	saveConf.setKey("display", "bchannel", key);
	sprintf(key, "%i", (int)display->getCharge()); 
	saveConf.setKey("display", "charge", key);
	sprintf(key, "%i", (int)display->getCaller()); 
	saveConf.setKey("display", "caller", key);
	sprintf(key, "%i", (int)display->getList()); 
	saveConf.setKey("display", "list", key);
	
	/* ports 1-4 */
	for (i=1; i<=4; i++) {
		if (i!=4)
			sprintf(portname, "port %i", i);
		else
			sprintf(portname, "pc");
		
		sprintf(key, "%i", (int)port[i]->getAuthorization()); 
		saveConf.setKey(portname, "authorization", key);
		saveConf.setKey(portname, "MSN", port[i]->getMSN());
		if (i!=4) {
			sprintf(key, "%i", (int)port[i]->getCallWaiting()); 
			saveConf.setKey(portname, "call waiting", key);
			sprintf(key, "%i", (int)port[i]->getDialTone()); 
			saveConf.setKey(portname, "dial tone", key);
			sprintf(key, "%i", (int)port[i]->getBusyTone()); 
			saveConf.setKey(portname, "busy tone", key);
		}
		sprintf(key, "%i", (int)port[i]->getFactor());
		saveConf.setKey(portname, "factor", key);
		sprintf(key, "%i", (int)port[i]->getMSNSuppression());
		saveConf.setKey(portname, "suppression", key);
	}
	
	/* collective call */
	for (i=1; i<=4; i++) {
		saveConf.setKey("collective", "MSN", collective->getMSN());
		if (i!=4)
			sprintf(portname, "port %i", i);
		else
			sprintf(portname, "pc");
		sprintf(key, "%i", (int)collective->getPort(i)); 
		saveConf.setKey("collective", portname, key);
	}
	sprintf(key, "%i", (int)collective->getMSNSuppression());
	saveConf.setKey("collective", "suppression", key);
	
	if (!(saveConf.writeRcFile(true))) {
		QMessageBox::warning(tabdialog, "Speichern", "Konnte Konfiguration nicht speichern!");
	}
}

void QTSD::saveAsConfiguration(void) {
	
	int len;
	
	QString s(QFileDialog::getSaveFileName(getenv("HOME"), "*.sdc", tabdialog));
	if (s==NULL) {
		return;
	}
	len=s.length();
	if (s.find(".sdc", len-4)==-1) {  /* append suffix */
		s.append(".sdc");
	}
	strcpy(configFile, s);
	saveConfiguration();
	setCaption("QtDragon - "+s);
}

void QTSD::openConfiguration(void) {

	char portname[10];
	int i;
	
	QString s(QFileDialog::getOpenFileName(getenv("HOME"), "*.sdc", tabdialog));
	if (s==NULL) {
		return;
	}
	strcpy(configFile, s);
	rcFile openConf(s);
	if (!(openConf.readRcFile(true))) {
		QMessageBox::warning(tabdialog, "ffnen", "Konnte Datei nicht ffnen!");
		return;
	}
	if (!(openConf.isFileValid())) {
		QMessageBox::warning(tabdialog, "ffnen", "Keine gltige Konfigurationsdatei!");
		return;
	}
	
	/* adapter */
	adapter->setAuthorization((tAmtsBerechtigung)atoi(openConf.getKey(
		"adapter", "authorization")));
	adapter->setCountry(atoi(openConf.getKey("adapter", "country")));
	adapter->setAreaCode(openConf.getKey("adapter", "area code"));
	adapter->setFactor(atoi(openConf.getKey("adapter", "factor")));
	
	/* display */
	display->setLanguage((tLanguage)atoi(openConf.getKey("display",
		"language")));
	display->setMSN((tMSN)atoi(openConf.getKey("display", "MSN")));
	display->setBChannel((tbchannel)atoi(openConf.getKey("display",
		"bchannel")));
	display->setCharge((bool)atoi(openConf.getKey("display", "charge")));
	display->setCaller((bool)atoi(openConf.getKey("display", "caller")));
	display->setList((bool)atoi(openConf.getKey("display", "list")));
	
	/* ports 1-4 */
	for (i=1; i<=4; i++) {
		if (i!=4)
			sprintf(portname, "port %i", i);
		else
			sprintf(portname, "pc");
		port[i]->setAuthorization((tAmtsBerechtigung)atoi(openConf.getKey
			(portname, "authorization")));
		port[i]->setMSN(openConf.getKey(portname, "MSN"));
		if (i!=4) {
			port[i]->setCallWaiting((bool)atoi(openConf.getKey(
				portname, "call waiting")));
			port[i]->setDialTone((bool)atoi(openConf.getKey(
				portname, "dial tone")));
			if (openConf.getKey(portname, "suppression")!=NULL) {
				port[i]->setMSNSuppression((bool)atoi(openConf.getKey(
					portname, "suppression")));
			}
			if (openConf.getKey(portname, "busy tone")!=NULL) {
				port[i]->setBusyTone((tBusyTone)atoi(openConf.getKey
					(portname, "busy tone")));	
			}
		}
		port[i]->setFactor(atoi(openConf.getKey(portname, "factor")));
	}
	
	/* collective call */
	collective->setMSN(openConf.getKey("collective", "MSN"));
	for (i=1; i<=4; i++) {
		if (i!=4)
			sprintf(portname, "port %i", i);
		else
			sprintf(portname, "pc");
		collective->setPort(i, (bool)atoi(openConf.getKey(
			"collective", portname)));	
	}
	if (openConf.getKey("collective", "suppression")!=NULL) {
		collective->setMSNSuppression((bool)atoi(openConf.getKey(
					"collective", "suppression")));
	}
	setCaption("QtDragon - "+s);
	resetStatusChanged();
}

void QTSD::clearCallerList(void) {
	if (QMessageBox::warning(tabdialog, "Anrufliste lschen", "Wollen Sie die Anrufliste\n" 
		"wirklich lschen?", "OK", "Abbruch", NULL, 1, 1)==0) {	
		userEventEnabled(FALSE);
		emit writingConfig("Lsche Anrufliste...");
		SD->vClearCallList();
		emit doneConfig("Bereit.");
		userEventEnabled(TRUE);
	}
}

void QTSD::resetCounter(void) {
	
	int i;
	
	if (QMessageBox::warning(tabdialog, "Alle Gebhren rcksetzen", "Wollen Sie wirklich alle" 
		"\nGebhrenzhler zurcksetzen?", "OK", "Abbruch", NULL, 1, 1)==0) {	
		emit writingConfig("Setze alle Gebhren zurck...");
		userEventEnabled(FALSE);
	
		/* clear counters */
		for (i=1; i<=4; i++)
			SD->vClearSummenzaehlerPort(i);
		SD->vClearSummenzaehlerPBX();
		/* read new status */
		adapter->setCounter(SD->iGetSummenzaehlerPBX());
		for (i=1; i<=4; i++)
			port[i]->setCounter(SD->iGetSummenzaehlerPort(i));
		emit doneConfig("Bereit.");
		userEventEnabled(TRUE);
	}
}

void QTSD::resetStatusChanged(void) {
	
	int i;
	
	#ifdef _DEBUG
	printf("resetting widget status...\n");
	#endif
	adapter->resetStatusChanged();
	display->resetStatusChanged();
	for (i=1; i<=4; i++)
		port[i]->resetStatusChanged();
	collective->resetStatusChanged();
}

void QTSD::showStatus(const char *message) {
	statusbar->message(message);
	statusbar->repaint();
	qApp->processEvents();
}

void QTSD::newSetup(void) {
	
	detectdlg *ddialog;
	
	ddialog=new detectdlg(tabdialog);
	ddialog->setupWin(0, SD->iGetBps(), getDeviceName());
	ddialog->resetStatusChanged();
	if (ddialog->exec()) { /* if ok pressed */
		repaint(); /* repaints main window before doing anything else */
		qApp->processEvents();
		if (ddialog->isBpsChanged()) {
			emit writingConfig("Setze neue Bitrate...");
			userEventEnabled(FALSE);
			SD->vChangeBpsPermanently(ddialog->getBps());
			emit doneConfig("Bereit.");
			userEventEnabled(TRUE);
		}
	}
	delete ddialog;
}

void QTSD::ISDNNetDialog(void) {
	
	netDlg *netdialog=new netDlg(tabdialog);
	
	qApp->setOverrideCursor(waitCursor);
	netdialog->setISDNNet(SD->xGetISDNNet());
	netdialog->setMSNTrans(SD->xGetMSNTransmission());
	netdialog->setService(SD->xGetAnalogDienstkennung());
	netdialog->resetStatusChanged();
	qApp->restoreOverrideCursor();
	netdialog->exec();
	if (netdialog->result()) {
		userEventEnabled(FALSE);
		if (netdialog->isISDNNetChanged()) {
			#ifdef _DEBUG
			printf("writing ISDN net/netdialog...\n");
			#endif
			SD->vSetISDNNet(netdialog->getISDNNet());
		}
		if (netdialog->isMSNTransChanged()) {
			#ifdef _DEBUG
			printf("writing MSN transmission/netdialog...\n");
			#endif
			SD->vSetMSNTransmission(netdialog->getMSNTrans());
		}
		if (netdialog->isServiceChanged()) {
			#ifdef _DEBUG
			printf("writing service id/netdialog...\n");
			#endif
			SD->vSetAnalogDienstkennung(netdialog->getService());
		}
	}
	userEventEnabled(TRUE);
	delete netdialog;	
}

void QTSD::diversionsDialog(void) {
	divDialog=new divDlg(tabdialog);
	
	updateDiversionsList();
	connect(divDialog, SIGNAL(deactivateAll()), SLOT(diversionDeactivateAll()));
	connect(divDialog, SIGNAL(activate()), SLOT(diversionActivate()));
	connect(divDialog, SIGNAL(deactivate()), SLOT(diversionDeactivate()));
	divDialog->exec();
}

void QTSD::diversionActivate(void) {
	emit writingConfig("Aktiviere Rufumleitung...");
	#ifdef _DEBUG
	printf("activating diversion: %s -> %s...\n", divDialog->getSelectedMSN(),
		divDialog->getSelectedID());
	#endif
	divDialog->repaint();
	qApp->processEvents();
	userEventEnabled(FALSE);
	switch (divDialog->getSelectedType()) {
		case divOff:
			break;
		case divCFU:
			SD->vSetRufumleitungCFU(divDialog->getSelectedMSN(),
				divDialog->getSelectedID());
			SD->vRufumleitungCFUOn(divDialog->getSelectedMSN());
			break;
		case divCFNR:
			SD->vSetRufumleitungCFNR(divDialog->getSelectedMSN(),
				divDialog->getSelectedID());
			SD->vRufumleitungCFNROn(divDialog->getSelectedMSN());
			break;
		case divCFB:
			SD->vSetRufumleitungCFB(divDialog->getSelectedMSN(),
				divDialog->getSelectedID());
			SD->vRufumleitungCFBOn(divDialog->getSelectedMSN());
			break;
		case divCFNRCFB:
			SD->vSetRufumleitungCFNR(divDialog->getSelectedMSN(),
				divDialog->getSelectedID(0));
			SD->vRufumleitungCFNROn(divDialog->getSelectedMSN());
			SD->vSetRufumleitungCFB(divDialog->getSelectedMSN(),
				divDialog->getSelectedID(1));
			SD->vRufumleitungCFBOn(divDialog->getSelectedMSN());
			break;
		case divBK2:
			SD->vSetRufumleitungBK2(divDialog->getSelectedMSN(),
				divDialog->getSelectedID());
			SD->vRufumleitungBK2On(divDialog->getSelectedMSN());
			break;
	}
	divDialog->removeAllMSNs();
	updateDiversionsList();
	userEventEnabled(TRUE);
	emit doneConfig("Done.");
}

void QTSD::diversionDeactivate(void) {
	emit writingConfig("Deaktiviere Rufumleitung...");
	#ifdef _DEBUG
	printf("deactivating diversion: %s -> %s...\n", divDialog->getSelectedMSN(),
		divDialog->getSelectedID());
	#endif
	userEventEnabled(FALSE);
	
	switch (divDialog->getSelectedType()) {
		case divOff:
			break;
		case divCFU:
			SD->vRufumleitungCFUOff(divDialog->getSelectedMSN());
			break;
		case divCFNR:
			SD->vRufumleitungCFNROff(divDialog->getSelectedMSN());
			break;
		case divCFB:
			SD->vRufumleitungCFBOff(divDialog->getSelectedMSN());
			break;
		case divCFNRCFB:
			SD->vRufumleitungCFNROff(divDialog->getSelectedMSN());
			SD->vRufumleitungCFBOff(divDialog->getSelectedMSN());
			break;
		case divBK2:
			SD->vRufumleitungBK2Off(divDialog->getSelectedMSN());
			break;
	}
	
	divDialog->removeAllMSNs();
	updateDiversionsList();
	userEventEnabled(TRUE);
	emit doneConfig("Done.");
}

void QTSD::diversionDeactivateAll(void) {
	emit writingConfig("Deaktiviere alle Rufumleitungen...");
	#ifdef _DEBUG
	printf("deactivating all diversions...\n");
	#endif
	userEventEnabled(FALSE);
	SD->vAllRufumleitungenOff();
	divDialog->removeAllMSNs();
	updateDiversionsList();
	userEventEnabled(TRUE);
	emit doneConfig("Bereit.");
}

void QTSD::remoteProgrammingDialog(void) {
	remoteDlg *remote=new remoteDlg(tabdialog);
	
	userEventEnabled(FALSE);
	remote->setPassword(SD->sGetFernprogPW().c_str());
	remote->setMSN(SD->sGetFernprogMSN().c_str());
	userEventEnabled(TRUE);
	remote->resetStatusChanged();
	
	if(remote->exec()) {
		update();
		qApp->processEvents();
		emit writingConfig("Schreibe Konfiguration: Fernprogrammierung...");
		userEventEnabled(FALSE);
		if (remote->isMSNChanged()) {
			if (strlen(remote->getMSN())!=0) {
				#ifdef _DEBUG
				printf("writing MSN/remote...\n");
				#endif
				SD->vSetFernprogMSN(remote->getMSN());
			}
			else {
				#ifdef _DEBUG
				printf("clearing MSN/remote...\n");
				#endif
				SD->vClearFernprogMSN();
			}
		}
		if (remote->isPasswordChanged()) {
			SD->vSetFernprogPW(remote->getPassword());
			#ifdef _DEBUG
			printf("writing password/remote...\n");
			#endif
		}
		emit doneConfig("Bereit.");
		userEventEnabled(TRUE);
		
	}
	delete remote;
}

void QTSD::aboutQtDragon(void) {

	char *aboutText="\
		QtDragon 0.7.3\n\
		\n\
		Author: Joachim Dorner\n\
		<jdorner@dillingen.baynet.de>\n\
		\n\
		Based on:\n\
		SD-class by Jan Willamowius\n\
		<jan@janhh.shnet.org>\n\
		\n\
		The latest version can be obtained at:\n\
		<http://www.dillingen.baynet.de/~jdorner>";
	
	QMessageBox::about(tabdialog, "Info ber QtDragon", aboutText);
}

void QTSD::createMenubar() {
	filemenu= new QPopupMenu();
	setupmenu= new QPopupMenu();
	viewmenu=new QPopupMenu();
	settingsmenu=new QPopupMenu();
	infomenu=new QPopupMenu();
	
	menubar->insertItem("&Datei",filemenu);
	menubar->setFrameStyle(QFrame::NoFrame);
	menubar->setMouseTracking(TRUE);
	filemenu->setStyle(WindowsStyle);
	filemenu->insertItem("ffnen", this, SLOT(openConfiguration()));
	filemenu->insertItem("Speichern", this, SLOT(saveConfiguration()));
	filemenu->insertItem("Speichern unter...", this, SLOT(saveAsConfiguration()));
	filemenu->insertSeparator();
	filemenu->setMouseTracking(TRUE);
	filemenu->insertItem("&Beenden", this, SLOT(quitSelected()));
	
	menubar->insertItem("&Konfiguration",setupmenu);
	setupmenu->setStyle(WindowsStyle);
	setupmenu->insertItem("Neu einlesen", this, SLOT(readConfiguration()));
	setupmenu->insertItem("Anlage programmieren", this, SLOT(writeConfiguration()));
	setupmenu->insertSeparator();
	setupmenu->insertItem("Anrufliste lschen", this, SLOT(clearCallerList()));
	setupmenu->insertItem("Alle Gebhren rcksetzen", this, SLOT(resetCounter()));
	setupmenu->insertSeparator();
	setupmenu->insertItem("Allgemein...", this, SLOT(ISDNNetDialog()));
	setupmenu->insertItem("Rufumleitungen...", this, SLOT(diversionsDialog()));
	setupmenu->setMouseTracking(TRUE);
	
	menubar->insertItem("&Ansicht", viewmenu);
	viewmenu->setStyle(WindowsStyle);
	viewmenu->setCheckable(TRUE);
	viewmenu->insertItem("Symbolleiste");
	viewmenu->connectItem(0, this , SLOT(toolbarToggled()));
	viewmenu->insertItem("Statusleiste");
	viewmenu->connectItem(1, this , SLOT(statusbarToggled()));
	viewmenu->setItemChecked(0, TRUE);
	viewmenu->setItemChecked(1, TRUE);
	viewmenu->setMouseTracking(TRUE);
	
	menubar->insertItem("&Einstellungen", settingsmenu);
	settingsmenu->insertItem("Fernprogrammierung", this, SLOT(remoteProgrammingDialog()));
	settingsmenu->insertItem("Anlagen-Setup", this, SLOT(newSetup()));
	settingsmenu->setStyle(WindowsStyle);
	settingsmenu->setMouseTracking(TRUE);
	
	menubar->insertItem("&Info", infomenu);
	infomenu->insertItem("Info ber QtDragon...", this, SLOT(aboutQtDragon()));
	infomenu->setStyle(WindowsStyle);
	infomenu->setMouseTracking(TRUE);
}

void QTSD::createToolbar(void) {
	
	QToolButton *fileOpen, *fileSave;
	QPixmap openIcon(fileopen), saveIcon(filesave);
	QToolButton *SDWrite, *SDRead;
	QPixmap writeIcon(sdwrite), readIcon(sdread);
	//QToolButton *filePrint;
	QPixmap printIcon(fileprint);
	QToolButton *divButton;
	QPixmap divIcon(diversion);
	QToolButton *helpButton;
	QPixmap helpIcon(help);

	toolbar=new QToolBar(this);
	
	fileOpen=new QToolButton(openIcon, "Konfigurationsdatei ffnen", 0,
		this, SLOT(openConfiguration()), toolbar);
	fileOpen->setFixedWidth(TOOL_BUTTON_WIDTH);
	fileSave=new QToolButton(saveIcon, "Konfiguration speichern", 0,
		this, SLOT(saveConfiguration()), toolbar);
	fileSave->setFixedWidth(TOOL_BUTTON_WIDTH);
	toolbar->setStyle(WindowsStyle);
	
	toolbar->addSeparator();
	SDRead=new QToolButton(readIcon, "Neu einlesen", 0,
		this, SLOT(readConfiguration()), toolbar);
	SDRead->setFixedWidth(TOOL_BUTTON_WIDTH);
	SDWrite=new QToolButton(writeIcon, "Anlage programmieren", 0,
		this, SLOT(writeConfiguration()), toolbar);
	SDWrite->setFixedWidth(TOOL_BUTTON_WIDTH);
	
	/*toolbar->addSeparator();
	filePrint=new QToolButton(printIcon, "Print Configuration", 0,
		this, SLOT(aboutQtDragon()), toolbar);
	filePrint->setFixedWidth(TOOL_BUTTON_WIDTH);*/
	
	toolbar->addSeparator();
	divButton=new QToolButton(divIcon, "Rufumleitungen", 0,
		this, SLOT(diversionsDialog()), toolbar);
	divButton->setFixedWidth(TOOL_BUTTON_WIDTH);
	
	toolbar->addSeparator();
	helpButton=new QToolButton(helpIcon, "Info", 0, 
		this, SLOT(aboutQtDragon()), toolbar);
	helpButton->setFixedSize(TOOL_BUTTON_WIDTH, TOOL_BUTTON_HEIGHT);
	
	QWidget *tmp=new QWidget(toolbar);
	toolbar->setStretchableWidget(tmp); 
	setRightJustification(TRUE); /* stretches toolbar to right edge */
	setDockEnabled(Right, TRUE);
}

void QTSD::createTabdialog(void) {
	
	int i;
	char label[10];
	
	/* add adapter tab */
	adapter=new tabAdapter(); 
	tabdialog->addTab(adapter, "Anlage"); 
	
	/*add display tab */
	display=new tabDisplay();
	tabdialog->addTab(display, "Display"); 

	/* add port tabs */
	for (i=1; i<=4; i++) {
		if (i!=4)
			sprintf(label, "Port %i", i);
		else 
			sprintf(label, "PC");
		port[i]=new tabPort();
		tabdialog->addTab(port[i], label); 
	}
	port[4]->setPort4(TRUE); /* removes dial tone and call waiting */
	
	/* add collective call tab */
	collective=new tabCollective();
	tabdialog->addTab(collective, "Sammelruf");
	
	/* add MSN tab */
	MSN=new tabMSN();
	tabdialog->addTab(MSN, "MSN berblick");	
}

void QTSD::userEventEnabled(bool val) {
	
	int current, i;
	
	menubar->setEnabled(val);
	current=tabdialog->currentTab();
	
	/* to disable the toolbuttons, first get all childs of toolbar */
	QObjectList *tmpList=(QObjectList*)toolbar->queryList("QToolButton");
	QObjectListIt it(*tmpList);
	/* iterate over the list and disable the buttons */
	if (it.toFirst()!=NULL) {
		while(it.current()!=NULL) {
			((QToolButton*)it.current())->setEnabled(val);
			++it;
		}
	}
	for (i=0; i<8; i++) {
		if (i!=current) /* this leaves current tab on top */
			tabdialog->setTabEnabled(i,val);
	}
	tabdialog->setTabEnabled(current, val);
	if (val)
		qApp->restoreOverrideCursor();
	else
		qApp->setOverrideCursor(waitCursor);
	closeEventAccepted=val;
}

void QTSD::closeEvent(QCloseEvent *e) {

	int i;
	bool changed=FALSE;

	if (closeEventAccepted) {
		if (adapter->isTabAdapterChanged())
			changed=TRUE;
		if (display->isTabDisplayChanged())
			changed=TRUE;
		for (i=1; i<=4; i++) {
			if (port[i]->isTabPortChanged())	
				changed=TRUE;
		}
		if (collective->isTabCollectiveChanged())
			changed=TRUE;
			
		if (changed) {
			switch(QMessageBox::warning(this, "Warnung",
				"Sollen die genderten Werte in die Anlage programmiert werden?"
				, "Ja", "Nein", "Abbruch", 0, 2)) {
				case 0: writeConfiguration();
					e->accept();
					break;
				case 1: e->accept();
					break;
				case 2: e->ignore();
			}
		}
		else
			e->accept();
	}
	else
		e->ignore();
}

void QTSD::updateDiversionsList(void) {

	int i;
	char ID[80];
	
	userEventEnabled(FALSE);
	emit writingConfig("Lese Rufumleitungsdaten...");
	for (i=1; i<=4; i++) {
		if (strlen(port[i]->getMSN())) {
			#ifdef _DEBUG
			printf("reading diversion/port %i...\n", i);
			#endif
			if(SD->bGetRufumleitungBK2Active(port[i]->getMSN())) {
				strcpy(ID,SD->sGetRufumleitungBK2(port[i]->getMSN()).c_str());
				divDialog->addMSN(port[i]->getMSN(),(divPort) (i-1),
					divBK2, ID);				
			
			}
			else if(SD->bGetRufumleitungCFUActive(port[i]->getMSN())) {
				strcpy(ID,SD->sGetRufumleitungCFU(port[i]->getMSN()).c_str());
				divDialog->addMSN(port[i]->getMSN(),(divPort) (i-1),
					divCFU, ID);				
			}
			else if((SD->bGetRufumleitungCFNRActive(port[i]->getMSN())) &&
					(SD->bGetRufumleitungCFBActive(port[i]->getMSN()))) {
				sprintf(ID, "%s/%s", SD->sGetRufumleitungCFNR(port[i]->getMSN()).c_str(),
					SD->sGetRufumleitungCFB(port[i]->getMSN()).c_str());
				divDialog->addMSN(port[i]->getMSN(),(divPort) (i-1),
					divCFNRCFB, ID);				
			}
			else if(SD->bGetRufumleitungCFBActive(port[i]->getMSN())) {
				strcpy(ID,SD->sGetRufumleitungCFB(port[i]->getMSN()).c_str());
				divDialog->addMSN(port[i]->getMSN(),(divPort) (i-1),
					divCFB, ID);			
			}
			else if(SD->bGetRufumleitungCFNRActive(port[i]->getMSN())) {
				strcpy(ID,SD->sGetRufumleitungCFNR(port[i]->getMSN()).c_str());
				divDialog->addMSN(port[i]->getMSN(),(divPort) (i-1),
					divCFNR, ID);				
			}
			else
				divDialog->addMSN(port[i]->getMSN(),(divPort) (i-1));
			
		}
	}
	if (strlen(collective->getMSN())) {
		#ifdef _DEBUG
		printf("reading diversion/collective...\n");
		#endif
		if(SD->bGetRufumleitungBK2Active(collective->getMSN()))	{
			strcpy(ID,SD->sGetRufumleitungBK2(collective->getMSN()).c_str());
			divDialog->addMSN(collective->getMSN(), Collective, divBK2, ID);				
		}
		else if(SD->bGetRufumleitungCFUActive(collective->getMSN())) {
			strcpy(ID,SD->sGetRufumleitungCFU(collective->getMSN()).c_str());
			divDialog->addMSN(collective->getMSN(), Collective, divCFU, ID);				
		}
		else if((SD->bGetRufumleitungCFNRActive(collective->getMSN())) &&
				(SD->bGetRufumleitungCFBActive(collective->getMSN()))) {
			sprintf(ID, "%s/%s", SD->sGetRufumleitungCFNR(collective->getMSN()).c_str(),
				SD->sGetRufumleitungCFB(collective->getMSN()).c_str());
			divDialog->addMSN(collective->getMSN(),Collective, divCFNRCFB, ID);				
		}
		else if(SD->bGetRufumleitungCFBActive(collective->getMSN())) {
			strcpy(ID,SD->sGetRufumleitungCFB(collective->getMSN()).c_str());
			divDialog->addMSN(collective->getMSN(),Collective, divCFB, ID);			
		}
		else if(SD->bGetRufumleitungCFNRActive(collective->getMSN())) {
			strcpy(ID,SD->sGetRufumleitungCFNR(collective->getMSN()).c_str());
			divDialog->addMSN(collective->getMSN(), Collective, divCFNR, ID);				
		}
		else
			divDialog->addMSN(collective->getMSN(), Collective);
	}
	userEventEnabled(TRUE);
	emit doneConfig("Bereit.");
}

void QTSD::statusbarToggled(void) {
	if (viewmenu->isItemChecked(1)) {
		viewmenu->setItemChecked(1, FALSE);
		statusbar->hide();
		setFixedHeight((height())-(statusbar->height()));
	}
	else {
		viewmenu->setItemChecked(1, TRUE);
		statusbar->show();
		setFixedHeight(height()+(statusbar->height()));
	}
	setFixedWidth(5+tabdialog->width()+5);
}

void QTSD::toolbarToggled(void) {
	if (viewmenu->isItemChecked(0)) {
		viewmenu->setItemChecked(0, FALSE);
		toolbar->hide();
		setFixedHeight((height())-(toolbar->height()));
	}
	else {
		viewmenu->setItemChecked(0, TRUE);
		toolbar->show();
		setFixedHeight(height()+(toolbar->height()));
	}
	setFixedWidth(5+tabdialog->width()+5);
}

void QTSD::tabMSNSelected(int id) {

	int i;
	char ports[20];
	
	setFocus(); /* resets focus whenever any tab was selected */
	if (id==7) { /* if MSN overview was selected */
		MSN->clearList();
		for (i=1; i<=4; i++) {
			if (strlen(port[i]->getMSN())) {
				sprintf(ports, "Port %i, PC", i);
				MSN->addItem(port[i]->getMSN(), ports, port[i]->getMSNSuppression());
			}
		}
		if (strlen(collective->getMSN()))
			MSN->addItem(collective->getMSN(), "Sammelruf, PC", collective->getMSNSuppression());
	}
}
		
