#ifndef QTSD_H
#define QTSD_H

#include <qmainwindow.h>
#include "sd.hpp"
#include "detectdlg.h"
#include "remotedlg.h"
#include "netdlg.h"
#include "divdlg.h"
#include "tabdialog.h"
#include "tabadapter.h"
#include "tabdisplay.h"
#include "tabport.h"
#include "tabmsn.h"
#include "tabcollective.h"
#include "rcfile.h"
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <string>
#include <qmessagebox.h>
#include <qfiledialog.h>
#include <qmenubar.h>
#include <qtoolbar.h>
#include <qpixmap.h>
#include <qtoolbutton.h>
#include <qstatusbar.h>
#include <sys/types.h>
#include <sys/stat.h>

#define TOOL_BUTTON_WIDTH 40
#define TOOL_BUTTON_HEIGHT 22

class QTSD : public QMainWindow {
	Q_OBJECT

public:
	QTSD(QWidget *parent=0, const char *name=0);
	~QTSD(void);
	
	void initSpeedDragon(int bps=0); 
	void setDeviceName(const char *device) { strcpy(gl_device, device); } 
	const char* getDeviceName(void) { return(gl_device); }
	
public slots:
	void readRcFile(void);
	void readConfiguration(void);
	void writeConfiguration(void);
	void saveConfiguration(void);
	void saveAsConfiguration(void);
	void openConfiguration(void);
	void clearCallerList(void);
	void resetCounter(void);
	void resetStatusChanged(void);
	void showStatus(const char*);
	void newSetup(void);
	void ISDNNetDialog(void);
	void diversionsDialog(void);
	void diversionActivate(void);
	void diversionDeactivate(void);
	void diversionDeactivateAll(void);
	void remoteProgrammingDialog(void);
	void aboutQtDragon(void);
	
signals:
	void readingConfig(const char* message);
	void writingConfig(const char* message);
	void doneConfig(const char *message);
	
private:
	ocSpeedDragon *SD;
	divDlg *divDialog;
	
	QMenuBar *menubar;
	QPopupMenu *filemenu, *setupmenu, *viewmenu, *settingsmenu, *infomenu;
	QStatusBar *statusbar;
	QToolBar *toolbar;
	
	tabDialog *tabdialog;
	tabAdapter *adapter;
	tabDisplay *display;
	tabPort *port[5];
	tabCollective *collective;
	tabMSN *MSN;
	
	char gl_device[30];
	char configFile[128];
	bool closeEventAccepted;
	
	void createMenubar(void);
	void createToolbar(void);
	void createTabdialog(void);
	void userEventEnabled(bool val);
	void closeEvent(QCloseEvent *e);
	void updateDiversionsList(void);
	
private slots:
	void quitSelected(void) { close(); }
	void statusbarToggled(void);
	void toolbarToggled(void);
	void tabMSNSelected(int id);
	
};

#endif
