#ifndef RCFILE_H
#define RCFILE_H

#include <iostream.h>
#include <fstream.h>
#include <string>
#include <stdlib.h>
#include <stdio.h>

#define MAX_LINE_LENGTH 80

enum tLine { tlComment, tlSection, tlKey, tlEmpty, tlUnknown };

typedef struct rcFileEntry {
	char key[MAX_LINE_LENGTH+1];
	char value[MAX_LINE_LENGTH+1];
	tLine type;
	rcFileEntry *nextEntry;
} rcFileEntry;

class rcFile {

public:
	rcFile::rcFile(const char *filename, const char *path=NULL);
	rcFile::~rcFile(void);
	
	bool readRcFile(bool checksum=false);
	void purgeRcFile(bool delNL=false, bool delComments=false,
		bool delUnknown=false);
	bool writeRcFile(bool checksum=false);
	char* getKey(const char *section, const char* key); 
	bool setKey(const char *section, const char *key,
		const char* value);
	bool delKey(const char* section, const char* key);
	bool setSection(const char *section, const char* newName); 
	bool delSection(const char *section, bool onlyIfEmpty=true);  
	int countKeys(const char *section);
	int countSections(void);
	int countEntries(void);
	bool moveSection(const char *section);
	char* moveNext(void);
	bool isFileValid(void); /* check if checksum is valid and if file isn't empty */
	
private:
	string iFileName;
	fstream iFile;
	bool iIsOpen;
	bool iIsValid;
	bool iEnteredNewSection;
	rcFileEntry *rootEntry, *cursor;
	
	bool openRcFile(bool readOnly=true);
	bool closeRcFile(void);
	char* readNextItem(void);
	void writeNextItem(const char* item);
	rcFileEntry* getSection(const char *section);
	char* getItem(const char *key, const rcFileEntry *entry=NULL);
	rcFileEntry* parseLine(const char *line);
	tLine lineType(const char *line);
	char* calcChecksum(void);
	
};

#endif
