#include "lang.h"
#include "tabadapter.h"

static const char *countries[] = {
	"Belgien (32)", "Dnemark (45)", "Deutschland (49)",
	"Finnland (358)", "Frankreich (33)", "Grobritannien (44)",
	"Irland (353)", "Italien (39)", "Lichtenstein (41)", 
	"Niederlande (31)", "Norwegen (47)", "sterreich (43)", 
	"Schweden (46)", "Schweiz (41)", 0
};

static const char *authorization[] = {
	0, "Halbamt", "Amt", "National", "International"
};

tabAdapter::tabAdapter(QWidget *parent, const char *name): QWidget(parent,name)
{
	
	QButtonGroup *g_auth;
	QGroupBox *g_charge;
	QLabel *l_country, *l_factor, *l_info, *l_area, *l_currency;
	QPushButton *b_reset;
	
	int x_country=50, y_country=30;
	int x_area=300, y_area=30;
	int x_auth=20 ,y_auth=90;
	int x_charge=190, y_charge=90;
	int x_info=15, y_info=20;
	int x_factor=15, y_factor=60;
	int x_rb=15, y_rb=20;
	
	setFixedSize(500, 220);
	
	/* Country */
	combo_country=new QComboBox(FALSE, this);
	combo_country->setStyle(WindowsStyle);
	l_country=new QLabel(combo_country, "Land:", this);
	l_country->setGeometry(x_country, y_country, 30, 25);
	combo_country->setGeometry(x_country+35, y_country,150,25);
	combo_country->setSizeLimit(14);
	//combo_country->insertStrList(country_code);
	connect(combo_country, SIGNAL(activated(int)), SLOT(areaCodeChanged()));
	
	/* Area Code */
	le_area=new QLineEdit(this);
	le_area->setStyle(WindowsStyle);
	l_area=new QLabel(le_area, "Vorwahl:", this);
	l_area->setGeometry(x_area, y_area, 50, 25);
	le_area->setGeometry(x_area+55, y_area, 80, 25);
	le_area->setMaxLength(MAX_AREA);
	connect(le_area, SIGNAL(textChanged(const char*)),SLOT(areaCodeChanged()));
	
	/* Authorization */
	g_auth=new QButtonGroup("Berechtigung", this);
	g_auth->setGeometry(x_auth, y_auth, 150, 110);
	connect(g_auth, SIGNAL(clicked(int)), SLOT(authorizationChanged()));
	rb_loc=new QRadioButton(authorization[2],g_auth);
	rb_loc->setGeometry(x_rb, y_rb, 100, 25);
	rb_nat=new QRadioButton(authorization[3], g_auth);
	rb_nat->setGeometry(x_rb, y_rb+30, 100, 25);
	rb_inat=new QRadioButton(authorization[4], g_auth);
	rb_inat->setGeometry(x_rb, y_rb+60, 100, 25);
	
	/* Charge info */
	g_charge=new QGroupBox("Gebhren", this);
	g_charge->setGeometry(x_charge, y_charge, 480-x_charge, 110);
	le_info=new QLineEdit(g_charge);
	l_info=new QLabel(le_info, "Zhler:", g_charge);
	l_info->setGeometry(x_info, y_info, 40,25);
	le_info->setGeometry(x_info+50, y_info, 50, 25);
	le_info->setEnabled(FALSE);
	connect(le_info, SIGNAL(textChanged(const char*)), SLOT(counterChanged()));
	l_currency= new QLabel("DM",g_charge);
	l_currency->setGeometry(x_info+110, y_info, 30,25);
	b_reset=new QPushButton("Rcksetzen", g_charge);
	b_reset->setGeometry(x_info+160, y_info, 50, 25);
	b_reset->setAutoResize(TRUE);
	connect(b_reset, SIGNAL(clicked()), SLOT(resetCounter()));
	
	le_factor=new QLineEdit(g_charge);
	le_factor->setStyle(WindowsStyle);
	l_factor=new QLabel(le_factor, "Faktor:", g_charge);
	l_factor->setGeometry(x_factor, y_factor, 40, 25);
	le_factor->setGeometry(x_factor+50, y_factor, 40, 25);
	le_factor->setMaxLength(MAX_FACTOR);
	connect(le_factor, SIGNAL(textChanged(const char*)), SLOT(factorChanged()));
}	resetStatusChanged();

void tabAdapter::setAuthorization(tAmtsBerechtigung type) {
	switch (type)  {
		case Halbamt:
		case Ortsamt:
			rb_loc->setChecked(TRUE);
			break;
		case National:
			rb_nat->setChecked(TRUE);
			break;
		case International:
			rb_inat->setChecked(TRUE);
	}
}

tAmtsBerechtigung tabAdapter::getAuthorization(void) {
	if (rb_loc->isChecked())
		return(Ortsamt);
	else if (rb_nat->isChecked())
		return(National);
	return(International);
}

void tabAdapter::setCountry(int country_code) {
	
	int code[]={ 32,45,49,358,33,44,353,39,41,31,47,43,46,41 };
	int index;
	
	combo_country->clear();
	combo_country->insertStrList(countries);
	
	for (index=0; index<14; index++)
		if (country_code==code[index]) {
			combo_country->setCurrentItem(index);
			break;
		}
} 
int tabAdapter::getCountry() {
	
	int code[]={ 32,45,49,358,33,44,353,39,41,31,47,43,46,41 };
	int index;
	
	index=combo_country->currentItem();
	return(code[index]);	
}

void tabAdapter::setAreaCode(const char *area) {
	le_area->setText(area);
}

const char* tabAdapter::getAreaCode() {
	
	const char *area;
	
	area=le_area->text();
	return(area);
}

void tabAdapter::setCounter(int charge) {
	
	char str_charge[20];
	float f_charge=charge;
	
	sprintf(str_charge,"%-3.2f", f_charge/100);
	le_info->setText(str_charge);
}

int tabAdapter::getCounter() {
	
	const char *str_charge;
	char str_charge_copy[20], b;
	int a,c;
	
	str_charge=le_info->text();
	sscanf(str_charge, "%i%c%i", &a,&b,&c);
	sprintf(str_charge_copy, "%i%i",a,c);
	return(atoi(str_charge_copy));
}

void tabAdapter::setFactor(int factor, int nachkomma) {
	
	char str_factor[20];
	
	if (nachkomma==0) {
		sprintf(str_factor,"%i", factor);
	}
	else {
		sprintf(str_factor,"%i.%i", factor, nachkomma);
	}
	le_factor->setText(str_factor);
}

int tabAdapter::getFactor() {

	const char *str_factor;
	char str_factor_copy[20]="\0";
	int i, len;
	
	str_factor=le_factor->text();
	len=strlen(str_factor);
	for (i=0; i<len; i++) {
		if (str_factor[i]=='.') {
			break;	
		}
		else { /* no period found */
			if (isdigit(str_factor[i])) 
				strncat(str_factor_copy, &str_factor[i], 1);
		}
	}
	return(atoi(str_factor_copy));
}

int tabAdapter::getNachkommaFactor(void) {
	const char *str_factor;
	char str_factor_copy[20]="\0";
	int len, digits=0, i=0;
	
	str_factor=le_factor->text();
	len=strlen(str_factor);
	
	while (str_factor[i]!='.') { 
		if (str_factor[i]==0) {
			return(0);
		}
		i++;
	} 
	for ( ; i<len; i++) {
		if ( (digits==2) || (str_factor[i]==0)) {
			break;
		}
		if (isdigit(str_factor[i])) {
			strncat(str_factor_copy, &str_factor[i], 1);
			digits++;
		}
	}
	return(atoi(str_factor_copy));
}

void tabAdapter::resetCounter(void) {
	setCounter(0);
}

void tabAdapter::resetStatusChanged(void) {
	Changed.AreaCode=FALSE;
	Changed.Counter=FALSE;
	Changed.Factor=FALSE;
	Changed.Authorization=FALSE;
}

bool tabAdapter::isTabAdapterChanged(void) {
	if (Changed.AreaCode || Changed.Counter || Changed.Factor || 
		Changed.Authorization)
		return(TRUE);
	else
		return(FALSE);
}
