#include "tabdialog.h"

tabDialog::tabDialog(QWidget *parent, const char *name) 
	: QWidget(parent,name, WStyle_Customize | WStyle_NoBorder )
{	
	layout=new QVBoxLayout(this, 0, -1, "layout");
	tabbar=new QTabBar(this); /* add tabar */
	stack=new QWidgetStack(this); /* add widget stack */
	
	connect(tabbar, SIGNAL(selected(int)), stack, SLOT(raiseWidget(int)));
	connect(tabbar, SIGNAL(selected(int)), SIGNAL(tabSelected(int)));
}
	
int tabDialog::addTab(QWidget *child, const char* label) {

	int id;

	QTab *tab=new QTab();
	tab->label=label;
	tab->enabled=TRUE;
	id=tabbar->addTab(tab);
	stack->addWidget(child, id);
	stack->setFixedSize(child->size());
	tabbar->setFixedSize(tabbar->sizeHint());
	if (id==0)
		stack->raiseWidget(0); /* show first child */
	updateLayout();
	return(id);	
}

void tabDialog::showTab(int id) {
	stack->raiseWidget(id);
}

void tabDialog::showTab(QWidget *child) {
	stack->raiseWidget(child);
}

int tabDialog::currentTab(void) {
	return(tabbar->currentTab());
}

void tabDialog::setTabEnabled(int id, bool val) {
	tabbar->setTabEnabled(id, val);
}

void tabDialog::setTabWidgetsEnabled(int id, bool val) {
	
	QWidget *top;
	
	if ((top=stack->widget(id))!=0) {
		QObjectList *list=top->queryList("QWidget");
		QObjectListIt it(*list);
		QObject *current;
		while((current=it.current())!=0) {
			((QWidget*)current)->setEnabled(val);
			++it;
		}
		delete list;
	}
	else 
		return;
	qApp->processEvents();
}

void tabDialog::updateLayout(void) {
	QBoxLayout *tmp=new QHBoxLayout();
	
	delete layout;
	layout=new QVBoxLayout(this, 0, -1, "layout");
	/* top margin */
	layout->addSpacing(6);
	
	/* tabbar */
	layout->addLayout(tmp, 0);
	tmp->addSpacing(6);
	tmp->addWidget(tabbar,0);
	//tmp->addStretch(1); /* only needed if dialog is resizable */
	tmp->addSpacing(8);
	
	/* tab pages (widget stack) */
	tmp=new QHBoxLayout();
	layout->addLayout(tmp, 0);
	tmp->addSpacing(7);
	tmp->addWidget(stack, 0);
	tmp->addSpacing(8);
	
	/* bottom margin */
	layout->addSpacing(6);
	
	layout->addStretch(1); /* this prevents the tab pages to grow */
	layout->activate();
}
void tabDialog::paintEvent( QPaintEvent * )
{
    if ( !tabbar )
	return;

	QPainter p;
	p.begin( this );
	
	QRect s( stack->geometry() );
	
	QCOORD t = s.top() - 1;
	QCOORD b = s.bottom() + 2;
	QCOORD r = s.right() + 2;
	QCOORD l = s.left() - 1;
    
	p.setPen( colorGroup().light() );
	// note - this line overlaps the bottom line drawn by QTabBar
	p.drawLine( l, t, r - 1, t );
	p.drawLine( l, t + 1, l, b );
	p.setPen( black );
	p.drawLine( r, b, l,b );
	p.drawLine( r, b-1, r, t );
	p.setPen( colorGroup().dark() );
	p.drawLine( l+1, b-1, r-1, b-1 );
	p.drawLine( r-1, b-2, r-1, t+1 );

	p.end();
}

