#include "tabdisplay.h"

static const char *language[] = {
	"Deutsch", "Englisch", "Franzsisch", "Italienisch", "Flmisch",
	"Dnisch", "Norwegisch", 0
};

tabDisplay::tabDisplay(QWidget *parent, const char *name): QWidget(parent,name)
{	
	QLabel *l_language;
	QGroupBox *g_selection;
	QButtonGroup *g_bchannel, *g_MSN;
	
	int x_language=50, y_language=30;
	int x_selection=20 , y_selection=70;
	int x_cb=15, y_cb=20;
	
	setFixedSize(500, 220);
	
	/*Language*/
	combo_language=new QComboBox(FALSE, this);
	combo_language->setStyle(WindowsStyle);
	l_language=new QLabel(combo_language, "Sprache:", this);
	l_language->setGeometry(x_language, y_language, 60, 25);
	combo_language->setGeometry(x_language+60, y_language, 100,25);
	combo_language->insertStrList(language);
	connect(combo_language, SIGNAL(activated(int)), SLOT(languageChanged()));
	
	/*Selection*/
	g_selection=new QGroupBox("Anzeige", this);
	g_selection->setGeometry(x_selection, y_selection, 460, 130);
	
	cb_MSN=new QCheckBox("MSNs: ",g_selection);
	cb_MSN->setGeometry(x_cb, y_cb, 80, 25);
	connect(cb_MSN, SIGNAL(clicked()), this, SLOT(MSN_toggled()));
	connect(cb_MSN, SIGNAL(clicked()), SLOT(MSNChanged()));
	g_MSN=new QButtonGroup(g_selection);
	g_MSN->setGeometry(x_cb+120, y_cb, 250,25);
	g_MSN->setFrameStyle(0);
	rb_diverted=new QRadioButton("umgeleitete", g_MSN);
	rb_diverted->setGeometry(10, 0, 100, 25);
	rb_all=new QRadioButton("alle", g_MSN);
	rb_all->setGeometry(150, 0, 100, 25);
	connect(g_MSN, SIGNAL(clicked(int)), SLOT(MSNChanged()));
	
	cb_bchannel=new QCheckBox("B-Kanal: ",g_selection);
	cb_bchannel->setGeometry(x_cb, y_cb+25, 80, 25);
	connect(cb_bchannel, SIGNAL(clicked()), this, SLOT(BChannel_toggled()));
	connect(cb_bchannel, SIGNAL(clicked()), SLOT(bChannelChanged()));
	g_bchannel=new QButtonGroup(g_selection);
	g_bchannel->setGeometry(x_cb+120, y_cb+25, 250,25);
	g_bchannel->setFrameStyle(0);
	rb_normal=new QRadioButton("normal", g_bchannel);
	rb_normal->setGeometry(10, 0, 100, 25);
	rb_advanced=new QRadioButton("erweitert", g_bchannel);
	rb_advanced->setGeometry(150, 0, 100, 25);
	connect(g_bchannel, SIGNAL(clicked(int)), SLOT(bChannelChanged()));
	
	cb_charge=new QCheckBox("Gebhren: ",g_selection);
	cb_charge->setGeometry(x_cb, y_cb+50, 80, 25);
	connect(cb_charge, SIGNAL(clicked()), SLOT(chargeChanged()));
	
	cb_list=new QCheckBox("Anrufliste",g_selection);
	cb_list->setGeometry(x_cb, y_cb+75, 80, 25);
	connect(cb_list, SIGNAL(clicked()), SLOT(listChanged()));
	
	cb_caller=new QCheckBox("Anrufer", g_selection);
	cb_caller->setGeometry(x_cb+130, y_cb+75, 80, 25);
	connect(cb_caller, SIGNAL(clicked()), SLOT(callerChanged()));
}

void tabDisplay::setLanguage(tLanguage type) {
	combo_language->setCurrentItem(type);
}

tLanguage tabDisplay::getLanguage(void) {
	return((tLanguage)(combo_language->currentItem()));
}

void tabDisplay::setMSN(tMSN type) {
	switch (type) {
		case MSN_Off:
			cb_MSN->setChecked(FALSE);
			break;
		case MSN_Diverted:
			rb_diverted->setEnabled(TRUE);
			rb_all->setEnabled(TRUE);
			cb_MSN->setChecked(TRUE);
			rb_diverted->setChecked(TRUE);
			break;
		case MSN_All:
			rb_diverted->setEnabled(TRUE);
			rb_all->setEnabled(TRUE);
			cb_MSN->setChecked(TRUE);
			rb_all->setChecked(TRUE);
			break;
	}
}

tMSN tabDisplay::getMSN(void) {
	if (cb_MSN->isChecked()) {
		if (rb_diverted->isChecked())
			return(MSN_Diverted);
		else return (MSN_All);
	}
	return(MSN_Off);			
}

void tabDisplay::setBChannel(tbchannel type) {
	switch (type) {
		case B_Off:
			cb_bchannel->setChecked(FALSE);
			break;
		case B_Normal:
			rb_normal->setEnabled(TRUE);
			rb_advanced->setEnabled(TRUE);
			cb_bchannel->setChecked(TRUE);
			rb_normal->setChecked(TRUE);
			break;
		case B_Advanced:
			rb_normal->setEnabled(TRUE);
			rb_advanced->setEnabled(TRUE);
			cb_bchannel->setChecked(TRUE);
			rb_advanced->setChecked(TRUE);
			break;
	}
}

tbchannel tabDisplay::getBChannel(void) {
	if (cb_bchannel->isChecked()) {
		if (rb_normal->isChecked())
			return(B_Normal);
		else return (B_Advanced);
	}
	return(B_Off);			
}

void tabDisplay::setCharge(bool val) {
	cb_charge->setChecked(val);
}

bool tabDisplay::getCharge() {
	return(cb_charge->isChecked());
}	

void tabDisplay::setCaller(bool val) {
	cb_caller->setChecked(val);
}

bool tabDisplay::getCaller() {
	return(cb_caller->isChecked());
}	

void tabDisplay::setList(bool val) {
	cb_list->setChecked(val);
}

bool tabDisplay::getList() {
	return(cb_list->isChecked());
}

void tabDisplay::MSN_toggled(void) {
	rb_diverted->setEnabled(cb_MSN->isChecked());	
	rb_all->setEnabled(cb_MSN->isChecked());
}

void tabDisplay::BChannel_toggled(void) {
	rb_normal->setEnabled(cb_bchannel->isChecked());	
	rb_advanced->setEnabled(cb_bchannel->isChecked());
}

void tabDisplay::resetStatusChanged(void) {
	Changed.Language=FALSE;
	Changed.BChannel=FALSE;
	Changed.MSN=FALSE;
	Changed.Charge=FALSE;
	Changed.Caller=FALSE;
	Changed.List=FALSE;
}

bool tabDisplay::isTabDisplayChanged(void) {
	if (Changed.Language || Changed.BChannel || Changed.MSN || 
		Changed.Charge || Changed.Caller || Changed.List)
		return(TRUE);
	else
		return(FALSE);
}
