#include "lang.h"
#include "tabport.h"

static const char *authorization[] = {
	0, "Halbamt", "Amt", "National", "International"
};

tabPort::tabPort(QWidget *parent, const char *name): QWidget(parent,name)
{
	
	QButtonGroup *g_auth;
	QGroupBox *g_charge;
	QLabel *l_MSN, *l_factor, *l_info, *l_currency;
	QPushButton *b_reset;
	
	int x_MSN=50, y_MSN=30;
	int x_auth=20 ,y_auth=90;
	int x_charge=230, y_charge=90;
	int x_info=15, y_info=20;
	int x_factor=15, y_factor=60;
	int x_rb=15, y_rb=20;
	int x_cb=280, y_cb=15;
	
	setFixedSize(500, 220);
	
	/*MSN*/
	le_MSN=new QLineEdit(this);
	le_MSN->setStyle(WindowsStyle);
	l_MSN=new QLabel(le_MSN, "MSN:", this);
	l_MSN->setGeometry(x_MSN, y_MSN, 30, 25);
	le_MSN->setGeometry(x_MSN+35, y_MSN, 100, 25);
	le_MSN->setMaxLength(MAX_MSN);
	connect(le_MSN, SIGNAL(textChanged(const char*)), SLOT(MSNChanged()));
	
	/*Misc*/
	cb_tone=new QCheckBox("Dauerwhlton (fr Faxbetrieb)", this);
	cb_tone->setGeometry(x_cb, y_cb, 200,25);
	connect(cb_tone, SIGNAL(clicked()), SLOT(dialToneChanged()));
	cb_waiting=new QCheckBox("Anklopfen", this);
	cb_waiting->setGeometry(x_cb, y_cb+20, 100,25);
	connect(cb_waiting, SIGNAL(clicked()), SLOT(callWaitingChanged()));
	cb_suppression=new QCheckBox("MSN Unterdrckung", this);
	cb_suppression->setGeometry(x_cb, y_cb+40, 150,25);
	connect(cb_suppression, SIGNAL(clicked()), SLOT(MSNSuppressionChanged()));
	
	/* Authorization */
	g_auth=new QButtonGroup("Berechtigung", this);
	g_auth->setGeometry(x_auth, y_auth, 120, 110);
	connect(g_auth, SIGNAL(clicked(int)), SLOT(authorizationChanged()));
	rb_no=new QRadioButton(authorization[1],g_auth);
	rb_no->setGeometry(x_rb, y_rb, 80, 25);
	g_auth->insert(rb_no);
	rb_loc=new QRadioButton(authorization[2], g_auth);
	rb_loc->setGeometry(x_rb, y_rb+20, 80, 25);
	g_auth->insert(rb_loc);
	rb_nat=new QRadioButton(authorization[3], g_auth);
	rb_nat->setGeometry(x_rb, y_rb+40, 80, 25);
	g_auth->insert(rb_nat);
	rb_inat=new QRadioButton(authorization[4], g_auth);
	rb_inat->setGeometry(x_rb, y_rb+60, 90, 25);
	g_auth->insert(rb_inat);
	
	/* Busy Tone */
	
	g_busy=new QButtonGroup("Besetztton", this);
	g_busy->setGeometry(x_auth+125, y_auth, 80, 110);
	connect(g_busy, SIGNAL(clicked(int)), SLOT(busyToneChanged()));
	rb_hard=new QRadioButton("hart",g_busy);
	rb_hard->setGeometry(x_rb, y_rb, 60, 25);
	g_busy->insert(rb_hard);
	rb_soft=new QRadioButton("weich", g_busy);
	rb_soft->setGeometry(x_rb, y_rb+30, 60, 25);
	g_busy->insert(rb_soft);
	rb_off=new QRadioButton("aus", g_busy);
	rb_off->setGeometry(x_rb, y_rb+60, 60, 25);
	g_busy->insert(rb_off);
	
	/* Charge info */
	g_charge=new QGroupBox("Gebhren", this);
	g_charge->setGeometry(x_charge, y_charge, 480-x_charge, 110);
	le_info=new QLineEdit(g_charge);
	l_info=new QLabel(le_info, "Zhler:", g_charge);
	l_info->setGeometry(x_info, y_info, 40,25);
	le_info->setGeometry(x_info+50, y_info, 50, 25);
	le_info->setEnabled(FALSE);
	connect(le_info, SIGNAL(textChanged(const char*)), SLOT(counterChanged()));
	l_currency= new QLabel("DM",g_charge);
	l_currency->setGeometry(x_info+110, y_info, 30,25);
	b_reset=new QPushButton("Rcksetzen", g_charge);
	b_reset->setGeometry(x_info+140, y_info, 80, 25);
	connect(b_reset, SIGNAL(clicked()), SLOT(resetCounter()));
	
	
	le_factor=new QLineEdit(g_charge);
	le_factor->setStyle(WindowsStyle);
	l_factor=new QLabel(le_factor, "Faktor:", g_charge);
	l_factor->setGeometry(x_factor, y_factor, 40, 25);
	le_factor->setGeometry(x_factor+50, y_factor, 40, 25);
	le_factor->setMaxLength(MAX_FACTOR);
	connect(le_factor, SIGNAL(textChanged(const char*)), SLOT(factorChanged()));
	resetStatusChanged();	
}

void tabPort::setPort4(bool val) {
	if (val) {
		cb_tone->hide();
		cb_waiting->hide();
		g_busy->hide();
	}
	else {
		cb_tone->show();
		cb_waiting->show();
		g_busy->show();
	}
}

void tabPort::setAuthorization(tAmtsBerechtigung type) {
	switch (type)  {
		case Halbamt:
			rb_no->setChecked(TRUE);
			break;
		case Ortsamt:
			rb_loc->setChecked(TRUE);
			break;
		case National:
			rb_nat->setChecked(TRUE);
			break;
		case International:
			rb_inat->setChecked(TRUE);
	}
}

tAmtsBerechtigung tabPort::getAuthorization(void) {
	if (rb_no->isChecked())
		return(Halbamt);
	else if (rb_loc->isChecked())
		return(Ortsamt);
	else if (rb_nat->isChecked())
		return(National);
	return(International);
}

void tabPort::setBusyTone(tBusyTone type) {
	switch (type)  {
		case HardBusy:
			rb_hard->setChecked(TRUE);
			break;
		case SoftBusy:
			rb_soft->setChecked(TRUE);
			break;
		case NoBusy:
			rb_off->setChecked(TRUE);
			break;
	}
}

tBusyTone tabPort::getBusyTone(void) {
	if (rb_hard->isChecked())
		return(HardBusy);
	else if (rb_off->isChecked())
		return(NoBusy);
	return(SoftBusy);
}

void tabPort::setMSN(const char *MSN) {
	le_MSN->setText(MSN);
}

const char* tabPort::getMSN(void) {
	return(le_MSN->text());
}

void tabPort::setDialTone(bool val) {
	cb_tone->setChecked(val);
}

bool tabPort::getDialTone(void) {
	return(cb_tone->isChecked());
}

void tabPort::setCallWaiting(bool val) {
	cb_waiting->setChecked(val);
}

bool tabPort::getCallWaiting(void) {
	return(cb_waiting->isChecked());
}

void tabPort::setMSNSuppression(bool val) {
	cb_suppression->setChecked(val);
}

bool tabPort::getMSNSuppression(void) {
	return(cb_suppression->isChecked());
}

void tabPort::setCounter(int charge) {
	
	char str_charge[20];
	float f_charge=charge;
	
	sprintf(str_charge,"%-3.2f", f_charge/100);
	le_info->setText(str_charge);
}

int tabPort::getCounter() {
	
	const char *str_charge;
	char str_charge_copy[20], b;
	int a,c;
	
	str_charge=le_info->text();
	sscanf(str_charge, "%i%c%i", &a,&b,&c);
	sprintf(str_charge_copy, "%i%i",a,c);
	return(atoi(str_charge_copy));
}

void tabPort::setFactor(int factor, int nachkomma) {
	
	char str_factor[20];
	
	if (nachkomma==0) {
		sprintf(str_factor,"%i", factor);
	}
	else {
		sprintf(str_factor,"%i.%i", factor, nachkomma);
	}
	le_factor->setText(str_factor);
}

int tabPort::getFactor() {
	
	const char *str_factor;
	char str_factor_copy[20]="\0";
	int i, len;
	
	str_factor=le_factor->text();
	len=strlen(str_factor);
	for (i=0; i<len; i++) {
		if (str_factor[i]=='.') {
			break;	
		}
		else { /* no period found */
			if (isdigit(str_factor[i])) 
				strncat(str_factor_copy, &str_factor[i], 1);
		}
	}
	return(atoi(str_factor_copy));
}

int tabPort::getNachkommaFactor(void) {
	const char *str_factor;
	char str_factor_copy[20]="\0";
	int len, digits=0, i=0;
	
	str_factor=le_factor->text();
	len=strlen(str_factor);
	
	while (str_factor[i]!='.') { 
		if (str_factor[i]==0) {
			return(0);
		}
		i++;
	} 
	for ( ; i<len; i++) {
		if ( (digits==2) || (str_factor[i]==0)) {
			break;
		}
		if (isdigit(str_factor[i])) {
			strncat(str_factor_copy, &str_factor[i], 1);
			digits++;
		}
	}
	return(atoi(str_factor_copy));
}

void tabPort::resetCounter(void) {
	setCounter(0);
}

void tabPort::resetStatusChanged(void) {
	MSNChanged();
	Changed.MSN=FALSE;
	Changed.Counter=FALSE;
	Changed.Factor=FALSE;
	Changed.Authorization=FALSE;
	Changed.BusyTone=FALSE;
	Changed.CallWaiting=FALSE;
	Changed.DialTone=FALSE;
	Changed.MSNSuppression=FALSE;
}

bool tabPort::isTabPortChanged(void) {
	if (Changed.MSN || Changed.Counter || Changed.Factor || 
		Changed.Authorization || Changed.CallWaiting ||
		Changed.DialTone || Changed.BusyTone || Changed.MSNSuppression)
		return(TRUE);
	else
		return(FALSE);
}

void tabPort::MSNChanged(void) {
	Changed.MSN=TRUE;
	if(strlen(getMSN()) > 0) {
		cb_suppression->setEnabled(TRUE);
	}
	else {
		cb_suppression->setEnabled(FALSE);
		cb_suppression->setChecked(FALSE);
	}
}
